/****************************************************************
**
**    Mandel Ultimativ  (for KDE)
**
****************************************************************/

// Megainclude
#include <qevent.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>

#include "basefrac.h"
#include "kmandel.h"
#include "mandelbrot.h"
#include "julia.h"
#include "l_mandel.h"
#include "l_julia.h"
#include "invmandel.h"
#include "ginger.h"
#include "galaxy.h"
#include "hop.h"
#include "chip.h"
#include "aboutdlg.h"
#include "dlgfractype.h"
#include "dlgfracparams.h"
#include "fracglobals.h"
#include "mandelapp.h"
#include "dlgpicset.h"
#include "dlgeditpal.h"
#include "dlgzoomsettings.h"
#include "parameter.h"
#include "dlgfilesettings.h"
#include "mandel_n.h"
#include "julia_n.h"
#include "triple.h"
#include "henon.h"
#include "barnsja.h"
#include "barnsb.h"
#include "barnsc.h"
#include "orbit3.h"
#include "wankel.h"
#include "quadrat.h"
#include "liapi.h"
#include "star.h"
#include "lorenz.h"
#include "genetic.h"
#include "pheonix.h"
#include "pocken.h"
#include "insel.h"
#include "mand4.h"
#include "mand4b.h"
#include "dlgparanim.h"
#include "dlgzoomanim.h"
#include "dlgjourney.h"
#include "sediment.h"
#include "newton.h"
#include "c_newton.h"
#include "lec211.h"
#include "lec212.h"
#include "lec213.h"
//#include "kmandel.moc"
#define UL UpdateInterval

#define MID_FILE     100   // Men File
#define MID_FILENEW  101   // File - New

const char* ZoomNames[] = {"Normal", "Normal", "Click", "Move"," Anim", "History", "Fast"}; // Texte fr Zoom-Modi
QColor ZoomRectColors[6];
KMandel *kmand; 
MandelApp* MApp;
KAboutData *aboutData;
int StartUp;    // Startup-Flag for Toolbar-Positioning-Problem....
int NoResize;   // ResizeEvent unterdrcken
int Opt_StatusbarBug=0; // Statusbar Bugfixing (0=off,1=on)

static const char *description = I18N_NOOP("kmandel");
static const char *version = "0.6";

static KCmdLineOptions options[] =
{
  { "s", I18N_NOOP("Fix statusbar-bug (no progess display)"), 0 },
  { 0, 0, 0 }
  // INSERT YOUR COMMANDLINE OPTIONS HERE
};

QDir MandelDir, ColorMapDir;

DrawWidget::DrawWidget( QWidget* parent, const char* name, WFlags f)
        : QWidget( parent, name, f )
{
long C;

   PicSizeX = 320;
   PicSizeY = 240;
   Berechnung = FALSE;   // keine Berechnung
   ImageReady = FALSE;   // kein Bild
   LoadImage  = FALSE;
   ZoomMode   = ZOOM_NONE;
   ZoomFactor = 5.0;
   ZoomFactorAnimation = 1.03;
   ZoomAnimationPics = 5;
   ZoomRectVisible = 0;  // kein Zoomrect sichtbar
   ColorDeep  = 8;       // 256 Farben
   NumColors  = 256;//xFFFFFFFF;
   SaveColorMapBool = FALSE;
   UpdateInterval = 50;
   UpdateLevel = 3;
   UpdateBool = TRUE;
   HiColorMode = 1;
   CalcMethod = CM_RECT;
   IterIncrement = 3;
   ZoomHistList = NULL;
   ZoomHistStart = NULL;
   ZoomHistEnd = NULL;
   ZoomHistFactor = 100.0;
   OrbitDisplayMode = 0;
   NoParamWarn = FALSE;
   ZoomRectColorSet = 0;
   HistList = NULL;
   HistStart = NULL;
   HistEnd = NULL;

   resize(PicSizeX,PicSizeY);

   C=NumColors;
   if ( (C>2) && (C<256) )  C=256;
   Image = new QImage(PicSizeX, PicSizeY, ColorDeep, C, QImage::BigEndian);
   Pix = new QPixmap();
//fprintf(stdout,"ImageBytes=%d\n",Image->numBytes());
   // Farbpalette setzen
   Image->setNumColors(256);       // Anzahl der Farben
   CreatePalette();
   InitPalette();

   Paint = new QPainter();
   Fractal = new MandelBrot(); //BaseFract();
   Fractal->Init(PicSizeX, PicSizeY);

   if ( parent )                           // has a child widget
      parent->installEventFilter( this ); // then install filter
//   connect(this, SIGNAL(FertigSig()), SLOT(Fertig()));
}

DrawWidget::~DrawWidget() {
   debug("DrawWidget::~DrawWidget");
}

void DrawWidget::SetPicSize(int SizeX, int SizeY)
{ 
   PicSizeX = SizeX;
   PicSizeY = SizeY;
   Fractal->SetSize(SizeX, SizeY);
   //ImageReady = FALSE;
   //repaint(TRUE);  // neue Berechnung erzwingen
}

void DrawWidget::resizeEvent(QResizeEvent*)
{
QImage *ImgTmp, *ImgOld;
long C;

   fprintf(stdout, "DrawWidget::ResizeEvent()  x= %d y=%d\n",width(),height());
   AbortCalc();
   SetPicSize(width(),height());  // neue Gre
   C=NumColors;
   if ( (C>2) && (C<256) )  C=256;
   ImgTmp = new QImage(PicSizeX, PicSizeY, ColorDeep, NumColors,QImage::BigEndian);
   ImgOld = Image;
   Image = ImgTmp;
   delete ImgOld;
   //Image = new QImage(PicSizeX, PicSizeY, ColorDeep, NumColors,QImage::BigEndian);
   InitPalette();
   Image->fill(0);
}

void DrawWidget::paintEvent(QPaintEvent*)
{
int p_active;
//debug("paintEvent");

   //fprintf(stdout, "DrawWidget::paintEvent\n");
   //fprintf(stdout, "Image x=%d, y=%d\n",Image->width(), Image->height());
   //fprintf(stdout, "Window x=%d, y=%d\n",PicSizeX, PicSizeY);

   if (ImageReady) {         // das Bild ist fertig, sofort anzeigen
      //fprintf(stdout, "Repaint: Image ready\n");
      p_active = Paint->isActive();
      if (!p_active)
         Paint->begin(this);
      //Pix = new QPixmap();
      Pix->convertFromImage(*Image, QPixmap::Auto);
      Paint->drawPixmap(0, 0, *Pix);
      //debug("UpdateImage");
      //delete Pix;
   
      if (!p_active)
         Paint->end();   
   } else
      CalcFractal(); 
   if (ImageReady) {
      //debug("FertigSig");
      emit FertigSig();
   }
}

void DrawWidget::mousePressEvent (QMouseEvent* mev) 
{
   //fprintf(stdout,"mousePressEvent\n");
/*   if (ZoomMode == ZOOM_FAST) { // "schneller" Zoom
      AbortCalc();
      Fractal->FastZoomIn(mev->x(), mev->y(), 1.05);
      update();
   } else
*/
   if (ZoomMode == ZOOM_HIST) {
      ZoomHistType* Ptr;
      int Zoom;
      
      if (ZoomHistStart != NULL) {
         Ptr = ZoomHistStart;
         Zoom = PicSizeX+1;
         while (Ptr != NULL) {
            if (mev->x()>=Ptr->x1)
            if (mev->x()<=Ptr->x2)
            if (mev->y()>=Ptr->y1)
            if (mev->y()<=Ptr->y2) {  // innerhalb des Rechtecks geklickt
               if (Zoom >= (Ptr->x2 - Ptr->x1)) { // dieses Rechteck ist kleiner
                  Zoom = Ptr->x2 - Ptr->x1;
                  ZoomHistList = Ptr;   // dann markieren
               }
            }
            Ptr = Ptr->Next;
         }
         ZoomHistShowRects();
      }
   } else
   if (ZoomMode==ZOOM_MOVE) {
      if (ZoomRectVisible==0) {  // nicht sichtbar
         ZoomRectVisible=1;
         ZoomRectX = PicSizeX/3;
         ZoomRectY = PicSizeY/3;
         ZoomX1 = PicSizeX/2;  // mittig
         ZoomY1 = PicSizeY/2;
         zxa=zya=bra=hoa=0;
      }
      mouseMoveEvent(mev);
   } else
   if (ZoomMode == ZOOM_ANIMATION) {
      if (ZoomRectVisible==0) {
         ZoomRectVisible = 1;
         zxa=zya=bra=hoa=0;
      }
      mouseMoveEvent(mev);
   } else
   if (ZoomMode == ZOOM_HIST) {
   } else
   {
      if (mev->button() == LeftButton) {
         //fprintf(stdout, "left button  b=%d\n",mev->button());
         ZoomMode = ZOOM_NORM;
         ZoomX1   = mev->x();
         ZoomY1   = mev->y();
         ZoomX2   = mev->x();
         ZoomY2   = mev->y();
         zxa=zya=bra=hoa=0;
      } else
      if (mev->button()==MidButton) {   // Point & Click
         ZoomX1   = mev->x();
         ZoomY1   = mev->y();
         ZoomMode = ZOOM_CLICK;
      }
   }
}
void DrawWidget::mouseReleaseEvent (QMouseEvent* mev) 
{
   //fprintf(stdout,"mouseReleaseEvent\n");
   if (ZoomMode == ZOOM_NORM) {
      ZoomX2   = mev->x();
      ZoomY2   = mev->y();
      if ( (mev->state() & ControlButton) != 0) {
         Fractal->ZoomOut(ZoomX1,ZoomY1, ZoomX2,ZoomY2);
         AbortCalc();
         update();
      } else
      if (Fractal->ZoomIn(ZoomX1,ZoomY1, ZoomX2,ZoomY2)) {
         //Berechnung=FALSE;
         //ImageReady=FALSE;
         AbortCalc();
         update();
      }
      ZoomMode = ZOOM_NONE;
      AddHist();
   } else
   if (ZoomMode == ZOOM_CLICK) {
      AbortCalc();
      if ( (mev->state() & ControlButton) != 0)
         Fractal->ZoomOut(ZoomX1, ZoomY1, ZoomFactor);
      else
         Fractal->ZoomIn(ZoomX1, ZoomY1, ZoomFactor);
      update();
      ZoomMode = ZOOM_NONE;
      AddHist();
   } else
   if (ZoomMode == ZOOM_MOVE) {
   } else
   if (ZoomMode == ZOOM_HIST) {
   }
}

void DrawWidget::mouseDoubleClickEvent (QMouseEvent* mev) {
   if (ZoomMode == ZOOM_HIST) {
      ZoomHistType* Ptr;
      int Zoom;
      QString qs(ZoomHistList->FileName);
      printf("qs=%s\n",qs.data());
      
      
      if (ZoomHistStart != NULL) {
         Ptr = ZoomHistStart;
         Zoom = PicSizeX+1;
         while (Ptr != NULL) {
            if (mev->x()>=Ptr->x1)
            if (mev->x()<=Ptr->x2)
            if (mev->y()>=Ptr->y1)
            if (mev->y()<=Ptr->y2) {  // innerhalb des Rechtecks geklickt
               if (Zoom >= (Ptr->x2 - Ptr->x1)) { // dieses Rechteck ist kleiner
                  Zoom = Ptr->x2 - Ptr->x1;
                  ZoomHistList = Ptr;   // dann markieren
               }
            }
            Ptr = Ptr->Next;
         }
         ZoomHistDeleteList();
         ZoomMode = ZOOM_NONE;
         kmand->SetStatus("", SZ_ZOOM);
         kmand->LoadParamFile(&qs);
         AddHist();
      }
   } else
   if (ZoomMode == ZOOM_MOVE) {  // Berechnung starten
      if ( (mev->state() & ControlButton) != 0) {
         AbortCalc();
         Fractal->ZoomOut(ZoomX1-ZoomRectX/2,ZoomY1-ZoomRectY/2, 
                          ZoomX1+ZoomRectX/2,ZoomY1+ZoomRectY/2);
         update();
      } else {
         AbortCalc();
         Fractal->ZoomIn (ZoomX1-ZoomRectX/2,ZoomY1-ZoomRectY/2, 
                          ZoomX1+ZoomRectX/2,ZoomY1+ZoomRectY/2);
         update();
      }
      AddHist();
   } else
   if (ZoomMode == ZOOM_ANIMATION) {
      ZoomAnimation(mev->x(), mev->y());
   } else
   if (ZoomMode == ZOOM_HIST) {
   }
}

void DrawWidget::mouseMoveEvent (QMouseEvent* mev) 
{
int zx,zy,br,ho;
int p_active;          // Anzeige: Painter aktiv

   if (ZoomMode == ZOOM_NORM) {
      //fprintf(stdout,"mouseMoveEvent\n");
      ZoomX2   = mev->x();
      ZoomY2   = mev->y();
      zx = QMIN(ZoomX1,ZoomX2);  // Ecke links oben
      zy = QMIN(ZoomY1,ZoomY2);
      br = abs(ZoomX1-ZoomX2);   // Gre
      ho = abs(ZoomY1-ZoomY2);
      p_active = Paint->isActive();
      if (!p_active)
         Paint->begin(this);
      Paint->drawPixmap(zxa,zya,*Pix, zxa,zya, bra,hoa);
      zxa=zx; zya=zy; bra=br; hoa=ho;
      Paint->setPen(ZoomRectColors[2*ZoomRectColorSet]);
      Paint->drawRect(zx,zy, br,ho);
      if (!p_active)
         Paint->end();
   } else
   if (ZoomMode == ZOOM_MOVE) {
      if (ZoomRectVisible) {
         if ((mev->state() == LeftButton) ||
             (mev->button()== LeftButton)) {  // Verschieben
            ZoomX1 = mev->x();  // Mitte des rechtecks
            ZoomY1 = mev->y();
         } else
         if ((mev->state() == MidButton) ||
             (mev->button()== MidButton)) {        // Gre ndern
            ZoomRectX = abs(ZoomX1-mev->x());        // Breite magebend
            if (ZoomRectX<16) ZoomRectX=16;          // Gre begrenzen
            ZoomRectY = ZoomRectX*PicSizeY/PicSizeX; // Hhe angleichen
         }
         
         zx = ZoomX1-ZoomRectX/2;  // Ecke links oben
         zy = ZoomY1-ZoomRectY/2;
         br = ZoomRectX;           // Gre
         ho = ZoomRectY;

         p_active = Paint->isActive();
         if (!p_active)
            Paint->begin(this);
         Paint->drawPixmap(zxa,zya,*Pix, zxa,zya, bra,hoa);
         zxa=zx; zya=zy; bra=br; hoa=ho;
         Paint->setPen(ZoomRectColors[2*ZoomRectColorSet]);
         Paint->drawRect(zx,zy, br,ho);
         if (!p_active)
            Paint->end();
      }
   } else
   if (ZoomMode == ZOOM_ANIMATION) {
      if (ZoomRectVisible) {
         if ((mev->state() == LeftButton) ||
             (mev->button()== LeftButton)) {  // Verschieben
            ZoomX1 = mev->x();  // Mitte
            ZoomY1 = mev->y();
         }
         zx = ZoomX1;  // Ecke links oben
         zy = ZoomY1;
         br = 10;           // Gre
         ho = 10;

         p_active = Paint->isActive();
         if (!p_active)
            Paint->begin(this);
         Paint->drawPixmap(zxa-6,zya-6,*Pix, zxa-6,zya-6, bra+2,hoa+2);
         zxa=zx; zya=zy; bra=br; hoa=ho;
         Paint->setPen(ZoomRectColors[2*ZoomRectColorSet]);
         Paint->drawLine(zx-5,zy, zx+5,zy);
         Paint->drawLine(zx,zy-5, zx,zy+5);
         if (!p_active)
            Paint->end();
      }
   } else
   if (ZoomMode == ZOOM_HIST) {
   }
} 


void DrawWidget::Taste(QKeyEvent *kev) {
int zx,zy,br,ho;
int p_active;          // Anzeige: Painter aktiv
int Offs;              // Offset fr die Verschiebung

   //debug("Taste  %d",kev->key());
   if (kev->key() == Key_Z) {
      if (ZoomMode == ZOOM_HIST) // Im Hostory-Mode nicht switchen
         return;
      MenuZoomMode();  // switchen
      return;
   }
   if (kev->key() == Key_C) {  // Farben der Rechtecke setzen
      ZoomRectColorSet++;
      if (ZoomRectColorSet>2)
         ZoomRectColorSet=0;
   }
   if (kev->key() == Key_L) {  // Zoom-History-Mode
      //debug("l pressed");
      //debug("ZoomMode=%d",ZoomMode);
      if (ZoomMode==ZOOM_HIST) {
         //debug("ZOOM_HIST");
         ZoomMode = ZOOM_NONE;
         ZoomHistDeleteList();

         p_active = Paint->isActive();
         if (!p_active)
            Paint->begin(this);
         Paint->drawPixmap(0,0,*Pix, 0,0, PicSizeX,PicSizeY);
         if (!p_active)
            Paint->end();
      } else {
         MenuZoomMode();    // nur zum Rechteck lschen und Status aktualisieren
         ZoomMode = ZOOM_HIST;
         ZoomHistBuildList();
      }
      kmand->SetStatus(ZoomNames[ZoomMode], SZ_ZOOM);
   }
   // PgUp oder PgDn aktiviert RectMove-Zoommode
   if ((kev->key() == Key_Next) || (kev->key() == Key_Prior)) {
      if ( (ZoomMode == ZOOM_NORM) || (ZoomMode==ZOOM_NONE)) {
         ZoomMode = ZOOM_MOVE;
         kmand->SetStatus(ZoomNames[ZoomMode], SZ_ZOOM);
      }
   }
   if (ZoomMode == ZOOM_HIST) {
      switch (kev->key()) {
         case Key_Left:
         case Key_Up:
         case Key_Prior : ZoomHistList = ZoomHistList->Prev;
                          if (ZoomHistList == NULL)
                             ZoomHistList = ZoomHistEnd;
                          break;
         case Key_Right:
         case Key_Down:
         case Key_Next  : ZoomHistList = ZoomHistList->Next;
                          if (ZoomHistList == NULL)
                             ZoomHistList = ZoomHistStart;
                          break;
         case Key_Home:   ZoomHistList = ZoomHistStart;
                          break;
         case Key_End:    ZoomHistList = ZoomHistEnd;
                          break;
         case Key_Enter:
         case Key_Return: {
                          QString qs(ZoomHistList->FileName);
                          //debug("qs=%s",qs.data());
                          
                             ZoomHistDeleteList();
                             ZoomMode = ZOOM_NONE;
                             kmand->SetStatus("", SZ_ZOOM);
                             kmand->LoadParamFile(&qs);
                             AddHist();
                          }
                          break;
      }
      if (ZoomHistList!=NULL)
         ZoomHistShowRects();
   } else
   if (ZoomMode == ZOOM_MOVE) {
      if (ZoomRectVisible) {       // Zoomrect ist sichtbar
         if (kev->state() & ControlButton) // Verschiebungsgeschwindigkeit
            Offs = 10;
         else
            Offs = 1;
         switch( kev->key() ) {
            case Key_Next:   ZoomRectX += 10;
                             if (ZoomRectX > PicSizeX)
                                ZoomRectX = PicSizeX;
                             break;
            case Key_Prior:  ZoomRectX -= 10;
                             if (ZoomRectX < 16)
                                 ZoomRectX=16;
                             break;
            case Key_Down:   ZoomY1 += Offs;
                             if (ZoomY1 > PicSizeY)
                                ZoomY1=PicSizeY;
                             break;
            case Key_Up:     ZoomY1 -= Offs;
                             if (ZoomY1 < 0)
                                ZoomY1 = 0;
                             break;
            case Key_Left:   ZoomX1 -= Offs;
                             if (ZoomX1 < 0)
                                ZoomX1 = 0;
                             break;
            case Key_Right:  ZoomX1 += Offs;
                             if (ZoomX1 > PicSizeX)
                                ZoomX1 = PicSizeX;
                             break;
            case Key_Home:   ZoomX1 = PicSizeX/2;
                             ZoomY1 = PicSizeY/2;
                             ZoomRectX = PicSizeX;
                             ZoomRectY = PicSizeY;
                             break;
            case Key_Return:  // Nur wo 'Enter' draufsteht, ist auch Enter drin !                 
            case Key_Enter:  //printf("enter\n"); 
                             ZoomRectVisible = FALSE;
                             AbortCalc();
                             if (kev->state() & ControlButton)
                                Fractal->ZoomOut(ZoomX1-ZoomRectX/2,ZoomY1-ZoomRectY/2, 
                                                 ZoomX1+ZoomRectX/2,ZoomY1+ZoomRectY/2);
                             else
                                Fractal->ZoomIn (ZoomX1-ZoomRectX/2,ZoomY1-ZoomRectY/2, 
                                                 ZoomX1+ZoomRectX/2,ZoomY1+ZoomRectY/2);
                             update();
                             AddHist();
                             break;
            case Key_Escape :   ZoomRectVisible = FALSE;
            default: kev->ignore();  // forget it
         }
      } else
      {      // Zoomrect nicht sichtbar
         ZoomRectVisible = TRUE;
         ZoomX1 = PicSizeX/2;  // mittig
         ZoomY1 = PicSizeY/2;
         ZoomRectX = PicSizeX;   // volle Gre
         ZoomRectY = PicSizeY;
         zxa=zya=bra=hoa=0;
      }
      ZoomRectY = ZoomRectX*PicSizeY/PicSizeX;
      zx = ZoomX1-ZoomRectX/2;  // Ecke links oben
      zy = ZoomY1-ZoomRectY/2;
      br = ZoomRectX;           // Gre
      ho = ZoomRectY;

      p_active = Paint->isActive();
      if (!p_active)
         Paint->begin(this);
      Paint->drawPixmap(zxa,zya,*Pix, zxa,zya, bra,hoa);
      if (ZoomRectVisible) {
         zxa=zx; zya=zy; bra=br; hoa=ho;
         Paint->setPen(ZoomRectColors[2*ZoomRectColorSet]);
         Paint->drawRect(zx,zy, br,ho);
      }
      if (!p_active)
         Paint->end();
   } else
   if (ZoomMode == ZOOM_ANIMATION) {
      if (ZoomRectVisible) {
         if (kev->state() & ControlButton) // Verschiebungsgeschwindigkeit
            Offs = 10;
         else
            Offs = 1;
         switch( kev->key() ) {
            case Key_Down:   ZoomY1 += Offs;
                             if (ZoomY1 > PicSizeY)
                                ZoomY1=PicSizeY;
                             break;
            case Key_Up:     ZoomY1 -= Offs;
                             if (ZoomY1 < 0)
                                ZoomY1 = 0;
                             break;
            case Key_Left:   ZoomX1 -= Offs;
                             if (ZoomX1 < 0)
                                ZoomX1 = 0;
                             break;
            case Key_Right:  ZoomX1 += Offs;
                             if (ZoomX1 > PicSizeX)
                                ZoomX1 = PicSizeX;
                             break;
            case Key_Home:   ZoomX1 = PicSizeX/2;
                             ZoomY1 = PicSizeY/2;
                             break;
            case Key_Return:
            case Key_Enter:  ZoomAnimation(ZoomX1, ZoomY1);
            		     break;
         }
      } else {
         ZoomRectVisible = 1;
         zxa=zya=bra=hoa=0;
         ZoomX1 = PicSizeX >> 1;   // mittich
         ZoomY1 = PicSizeY >> 1;
      }
      zx = ZoomX1;  // Ecke links oben
      zy = ZoomY1;
      br = 10;           // Gre
      ho = 10;

      p_active = Paint->isActive();
      if (!p_active)
         Paint->begin(this);
      Paint->drawPixmap(zxa-6,zya-6,*Pix, zxa-6,zya-6, bra+2,hoa+2);
      if (ZoomRectVisible) {
         zxa=zx; zya=zy; bra=br; hoa=ho;
         Paint->setPen(ZoomRectColors[2*ZoomRectColorSet]);
         Paint->drawLine(zx-5,zy, zx+5,zy);
         Paint->drawLine(zx,zy-5, zx,zy+5);
      }
      if (!p_active)
         Paint->end();
   }
}

void DrawWidget::CalcFractal(void)
{
int fc;

//fprintf(stdout,"Calcfractal - Berechnung\n");
if (Berechnung) return;

//   if (StartUp) return;
   LoadImage=FALSE;
   PixCount = 0;
   kmand->SetStatus("", SZ_PROGRESS);

//debug("InsMode=%d  const=%d",Fractal->InsideMethod, INS_SOLIDCOLOR);

   fc = Fractal->GetFracClass();
   switch (fc) {
      case FC_FIELD : CalcMandel(); break;
      case FC_ORBIT : CalcOrbit(); break;
      case FC_IAO   : CalcImageAtOnce(); break;
   }
//   kmand->SetStatus(klocale->translate("ready"), SZ_PROGRESS);
   kmand->StatusZeile->changeItem(i18n("ready"), SZ_PROGRESS);
   update();
}

void DrawWidget::PutPixel(int x, int y, long Iter)
{
int b,x1;

   if (ColorDeep==8)
      *(Image->scanLine(y)+x) = Iter%256;
   else
   if (ColorDeep==1) {
      b = x%8;
      x1= x/8;
      Iter = Iter%NumColors;
      if (Iter)
         *(Image->scanLine(y)+x1) |= (128>>b);
      else
         *(Image->scanLine(y)+x1) &= ~(128>>b);
   } else {  // 32 Bit
         x1 = x<<2;
      switch (HiColorMode) {
         case 0 :  *(Image->scanLine(y)+x1)   = Iter & 255;   // Rot
                   *(Image->scanLine(y)+x1+1) = 0; //255-(Iter >> 8)&255; // grn
                   *(Image->scanLine(y)+x1+2) = 255-((Iter >> 16)&255);  // Blau
                   *(Image->scanLine(y)+x1+3) = 0; break;

         case 1 :  *(Image->scanLine(y)+x1)   = (char)(sin(Iter*0.05)*127+127);   // Rot
                   *(Image->scanLine(y)+x1+1) = (char)(-sin(Iter*0.03)*127+127); // grn
                   *(Image->scanLine(y)+x1+2) = (char)(sin(Iter*0.08)*127+127);  // Blau
                   *(Image->scanLine(y)+x1+3) = 0; break;
      }         
   }
   if ((PixCount % (UpdateInterval*PicSizeX))  == 0)
   // Status-Update nach xxx Zeilen
   if (Fractal->GetFracClass() == FC_FIELD) 
      ShowProgress();
}

void DrawWidget::ShowProgress(void) {
char Text[10];

   if (Fractal->GetFracClass() == FC_IAO) 
      sprintf(Text, "%3ld%%", Fractal->GetProgress());
   else
      sprintf(Text, "%3ld%%", (100*PixCount)/PixMax);
   kmand->SetStatus(Text, SZ_PROGRESS);
}

long DrawWidget::GetPixel(int x, int y)
{
int b,x1;

   if (ColorDeep==8)
      return *(Image->scanLine(y)+x);
   else if (ColorDeep==1) {
      b = x%8;
      x1= x/8;
      return (((*(Image->scanLine(y)+x1)) & (128>>b)) != 0);
   } else{
      x1 = x<<2;
      unsigned long b1,b2,b3,b4;
      b1 = *(Image->scanLine(y)+x1);
      b2 = *(Image->scanLine(y)+x1+1);
      b3 = *(Image->scanLine(y)+x1+2);
      b4 = *(Image->scanLine(y)+x1+3);
      return b1 + (b2*256) + (b3*65536);// + (b3<<24);
   }
}

int DrawWidget::TestRect(int x1, int x2, int y1, int y2) {
// Prft, ob alle Punkte auf dem Rechteck(x1y1, x2y2) die gleiche Farbe
// haben und gibt dann "true" (=1) zurck, sonst "False" (=0)
// Die Optimierung geht dahin, mgligst frh zu dem Ergebnis "False"
// zu kommen, da dieser Fall wohl am meisten eintritt
int Ret;
int Lauf;
int Pixel;
// return True;

   // die vier Eckpunkte zuerst vergleichen
   Ret = TRUE;
   Pixel = GetPixel(x1,y1);

   if (Pixel != GetPixel(x2,y1) )
      return TRUE;
   if (Pixel != GetPixel(x2,y2) )
      return TRUE;
   if (Pixel != GetPixel(x1,y2) )
      return TRUE;

   // waagerecht oben
   Lauf = x1+1;
   while ( Ret && (Lauf < x2) )
   {
      Ret = (GetPixel(Lauf,y1) == Pixel) &
        (GetPixel(Lauf,y2) == Pixel);
      Lauf++;
   }

   // senkrecht rechts
   Lauf = y1+1;
   while ( Ret && (Lauf < y2) )
   {
      Ret = (GetPixel(x1, Lauf) == Pixel) &
        (GetPixel(x2, Lauf) == Pixel);
      Lauf++;
   }
   // Alle Punkte auf dem Rahmen haben gleiche Farbe,
   // weitere Berechnung nicht ntig
   return !Ret;
}

void DrawWidget::PaintRect(int x1, int x2, int y1, int y2, int Level) {
// Recheck ausmalen
int i,j,X;
unsigned long Color;
   Color = GetPixel(x1,y1);
   x1++;y1++;
   for (i=x1;i<x2;i++)
   for (j=y1;j<y2;j++) {
      if (ColorDeep == 32) {
         X = i<<2;
         *(Image->scanLine(j)+X)   = (char)(Color % 256);   // Rot
         *(Image->scanLine(j)+X+1) = (char)((Color >> 8) % 256); // grn
         *(Image->scanLine(j)+X+2) = (char)((Color >> 16) % 256);  // Blau
         *(Image->scanLine(j)+X+3) = 0;
      } else
         PutPixel(i,j,Color);
      PixCount++;
   }

   if (Level<UpdateLevel)  {
      Pix->convertFromImage(*Image);
//      Paint->drawPixmap(0, 0, *Pix);
      Paint->drawPixmap(x1-1,y1-1,*Pix, x1-1,y1-1, x2-x1+2,y2-y1+2);
//      update();
   }
/*   if (Level<(UpdateLevel<<1)) {
     update();
     qApp->processEvents();
   }*/
}

// Waagerecht teilen
void DrawWidget::Calc2w (int x1, int x2, int y1, int y2, int Level) {
int i, Mitte, Lauf;

   if (!Berechnung) return;
   Mitte = (y2-y1)/2;
   Mitte = y1+Mitte;

   Lauf = 1;
   for (i=x1+1; i<x2; i++) {
      Lauf++;
      PutPixel(i, Mitte, Fractal->CalcPoint(i, Mitte));
      if (!Berechnung) return;
      PixCount++;
   }
   if (Mitte-1>y1)
   if (TestRect(x1,x2,y1,Mitte))
      Calc2s(x1,x2,y1,Mitte, Level+1);  // obere Seite
   else
      PaintRect(x1,x2,y1,Mitte,Level);
   if (!Berechnung) return;

   if (y2>Mitte+1)
   if (TestRect(x1,x2,Mitte,y2))
      Calc2s(x1,x2,Mitte,y2, Level+1);  // untere Seite
   else
      PaintRect(x1,x2,Mitte,y2,Level);

   if (Level==UpdateLevel)  {
      Pix->convertFromImage(*Image);
//      Paint->drawPixmap(0, 0, *Pix);
      Paint->drawPixmap(x1,y1,*Pix, x1,y1, x2-x1,y2-y1);
//debug("Height=%d",Image->height());
//     update();
   }
   if (Level==(UpdateLevel<<1)) {
//     update();
     qApp->processEvents();
   }
}

// senkrecht teilen
void DrawWidget::Calc2s (int x1, int x2, int y1, int y2, int Level) {
int i, Mitte, Lauf;
 
   if (!Berechnung) return;
   Mitte = (x2-x1)/2;
   Mitte = x1+Mitte;

   Lauf = 1;
   for (i=y1+1; i<y2; i++) {
      Lauf++;
      PutPixel(Mitte, i, Fractal->CalcPoint(Mitte, i));
      if (!Berechnung) return;
      PixCount++;
   }
   if (Mitte-1>x1)
   if (TestRect(x1,Mitte,y1,y2))
      Calc2w(x1,Mitte,y1,y2,Level+1);  // linke Seite
   else
      PaintRect(x1,Mitte,y1,y2,Level);
   if (!Berechnung) return;

   if (x2>Mitte+1)
   if (TestRect(Mitte,x2,y1,y2))
      Calc2w(Mitte,x2,y1,y2,Level+1);  // rechts Seite
   else
      PaintRect(Mitte,x2,y1,y2,Level);

   if (Level==UpdateLevel)  {
      Pix->convertFromImage(*Image);
//      Paint->drawPixmap(0, 0, *Pix);
      Paint->drawPixmap(x1,y1,*Pix, x1,y1, x2-x1,y2-y1);
//      update();
   }
   if (Level==(UpdateLevel<<1)) {
//      update();
      qApp->processEvents();
    }
}

void DrawWidget::Calc2(void) {
int i; 

   if (!UpdateBool) UpdateLevel=-1; else  // kein Update
   if (UL<50)  UpdateLevel = 4; else      // feines Update
   if (UL<100) UpdateLevel = 3; else      // mittleres Update
   if (UL<250) UpdateLevel = 2; else      // grobes Update
   UpdateLevel = 1;

   // Rand berechnen
   PixCount=1;
   for (i=0; i< PicSizeX-1; i++) {
      PutPixel(i, 0,       Fractal->CalcPoint(i, 0));
      PutPixel(i, PicSizeY-1, Fractal->CalcPoint(i, PicSizeY-1));
   }

   for (i=1; i< PicSizeY-1; i++) {
      PutPixel(0, i,       Fractal->CalcPoint(0, i));
      PutPixel(PicSizeX-1, i, Fractal->CalcPoint(PicSizeX-1, i));
   }
   PixCount = PixCount+PicSizeX+PicSizeX+PicSizeY+PicSizeY;
   

   Calc2s(0,PicSizeX-1,0,PicSizeY-1,0);

   if (!Berechnung) {     // Wurde die Berechnung von auen abgebrochen
                          // jetzt sofort beenden
      Paint->end();
      if (LoadImage) {
         ImageReady = TRUE;
         LoadImage = FALSE;
      } else {
         ImageReady=FALSE;
//         update();
      }
      //fprintf(stdout," ABORT\n");
      //return;
   }
}

void DrawWidget::AutoLogMap(void) {
int i,iter,min,m2;

   kmand->StatusZeile->changeItem("Frame", SZ_PROGRESS);
   qApp->processEvents();
   Fractal->AutoLogFlag = TRUE;

   // Rand oben
   min=Fractal->MaxIter;
   for (i=0; i<PicSizeX; i+=3) {
      iter = Fractal->CalcPoint(i,0);
      if (iter<min) min=iter;
   }
   m2 = PicSizeY >> 1;
   // mitte waag
   for (i=0; i<PicSizeX; i+=3) {
      iter = Fractal->CalcPoint(i,m2);
      if (iter<min) min=iter;
   }
   // unten
   for (i=0; i<PicSizeX; i+=3) {
      iter = Fractal->CalcPoint(i,PicSizeY-1);
      if (iter<min) min=iter;
   }
   // links
   for (i=0; i<PicSizeY; i+=3) {
      iter = Fractal->CalcPoint(0,i);
      if (iter<min) min=iter;
   }
   m2 = PicSizeX >> 1;
   // mitte senk
   for (i=0; i<PicSizeY; i+=3) {
      iter = Fractal->CalcPoint(m2,i);
      if (iter<min) min=iter;
   }
   // rechts
   for (i=0; i<PicSizeY; i+=3) {
      iter = Fractal->CalcPoint(PicSizeX-1,i);
      if (iter<min) min=iter;
   }
   Fractal->LogFirstColor = min;
//   Fractal->AutoLogColor = FALSE;
//   printf("Autologcolor=%d\n",min);
   Fractal->AutoLogFlag = FALSE;
}

void DrawWidget::CalcMandel(void)
{
int x,y;

   //fprintf(stdout,"CalcMandel %d  %d\n",Pix->width(),Pix->height());
   if (Berechnung) {
      //fprintf(stdout,"CalcMandel rejected\n");
      return;
   }
   if(!Paint->isActive()) {   // berschneiden sich zwei Paint-Aufrufe,
                              // den nachfolgenden ignorieren

      // Variablen fr Statusanzeige
      PixMax = PicSizeX*PicSizeY;
      PixCount=0;
      
      Paint->begin(this);
      Berechnung = TRUE;
      ImageReady=FALSE;
      //Pix=new QPixmap();

      Fractal->InitField();            // Field-Type init
      if (Fractal->AutoLogColor)
         AutoLogMap();                 // Berechnung der log. Farbpalette
      Fractal->AutoLogFlag=FALSE;
      if (CalcMethod == CM_RECT) {
         Calc2();
         if (!Berechnung) {
            return;
         }
      } else
      {
         for(y=0;y<PicSizeY;y++) {
            for(x=0;x<PicSizeX;x++) {
               if (!Berechnung) {     // Wurde die Berechnung von auen abgebrochen
                                      // jetzt sofort beenden
                  Paint->end();
                  if (LoadImage) {
                     ImageReady = TRUE;
                     LoadImage = FALSE;
                  } else {
                     ImageReady=FALSE;
                     update();
                  }
                  //fprintf(stdout," ABORT\n");
                  return;
               }
               PutPixel(x,y, Fractal->CalcPoint(x,y));
               PixCount++;
            }
            if (UpdateBool)
            if (y%UL == 0) {
               Pix->convertFromImage(*Image);
               Paint->drawPixmap(0,y/UL*UL-UL,*Pix, 0,y/UL*UL-UL, PicSizeX,UL);
            }

            qApp->processEvents();
         }
      }
      if (Berechnung) {
         Pix->convertFromImage(*Image);
         Paint->drawPixmap(0, 0, *Pix);
      }
      Paint->end();   

      //delete Pix;
   }  //else
      //fprintf(stdout,"Painter is active\n");
   Berechnung=FALSE;
   ImageReady = TRUE;
   //fprintf(stdout,"ImageReady=True\n");
}

void DrawWidget::CalcOrbit(void)
{
int Loops;  // Anzahl der Durchlufe
int Lauf;   // aktueller Durchlauf
int ix,iy;  //  Bildpunkt
int Color;  // Farbe fr den Punkt
double li=Fractal->li,re=Fractal->re,un=Fractal->un,ob=Fractal->ob;

   if(!Paint->isActive()) {   // berschneiden sich zwei Paint-Aufrufe,
                              // den nachfolgenden ignorieren

      //fprintf(stdout,"CalcOrbit\n");
      Fractal->OrbX = 0;
      Fractal->OrbY = 0;
      Fractal->OrbZ = 0;
      Fractal->InitOrbit();
      Loops = (PicSizeX+PicSizeY/10)*Fractal->MaxIter;
      PixMax=Loops;  // fr Statusanzeige
      PixCount=0;

      Berechnung = TRUE;
      ImageReady = FALSE;
      Color = rand();
      Paint->begin(this);
 
      for (Lauf=0; Lauf<Loops; Lauf++) {
         PixCount=Lauf;  // fr Statusanzeige
         if (Lauf % (Fractal->MaxIter<<4) == 0)
            ShowProgress();
         
         Fractal->CalcOrbit();

         ix = (int)((Fractal->OrbX-li)/(re-li)*PicSizeX);
         iy = (int)((Fractal->OrbY-un)/(ob-un)*PicSizeY);

         // Punkt setzen
         if (ix >= 0)

         if (ix < PicSizeX)
         if (iy >=0)
         if (iy < PicSizeY) {
           if (OrbitDisplayMode==1) {
             int PixVal = GetPixel(ix,iy);
             PutPixel(ix, iy, PixVal+1);
           } else
             PutPixel(ix, iy, Color);
         }

         if (Lauf%PicSizeY==0)   // Farbwechsel
            Color = rand();

         if (UpdateBool)
         if (Lauf%(UL*Fractal->MaxIter) == 0) {
            Pix->convertFromImage(*Image);
            Paint->drawPixmap(0,0,*Pix);
         }
         qApp->processEvents();

         if (!Berechnung) {     // Wurde die Berechnung von auen abgebrochen
                                // jetzt sofort beenden
            Paint->end();
            if (LoadImage) {
               ImageReady = TRUE;
               LoadImage = FALSE;
            } else {
               ImageReady=FALSE;
               update();
            }
            //fprintf(stdout," ABORT\n");
            return;
         }
      }//for()

      Pix->convertFromImage(*Image);
      Paint->drawPixmap(0, 0, *Pix);
      Paint->end();
      Berechnung = FALSE;
      ImageReady = TRUE;
      //fprintf(stdout,"Calcorbit: ende\n");
   }
}

void DrawWidget::CalcImageAtOnce(void)
// Ganzes Bild auf einmal barechnen
{
//fprintf(stdout,"ImageAtOnce-Start\n");
   if(!Paint->isActive()) {   // berschneiden sich zwei Paint-Aufrufe,
                              // den nachfolgenden ignorieren

      Fractal->InitIAO();
      
      Berechnung = TRUE;
      ImageReady = FALSE;
      Paint->begin(this);

      Fractal->CalcIAO();
      //FillRect(10,10,100,100,  (long)100);
      
/*      PutPixel(10,10, 100);
      PutPixel(11,10, 100);
      PutPixel(10,11, 100);
      PutPixel(11,11, 100);
*/      
         qApp->processEvents();

         if (!Berechnung) {     // Wurde die Berechnung von auen abgebrochen
                                // jetzt sofort beenden
            Paint->end();
            if (LoadImage) {
               ImageReady = TRUE;
               LoadImage = FALSE;
            } else {
               ImageReady=FALSE;
               update();
            }
            //fprintf(stdout," ABORT\n");
            return;
         }

      Pix->convertFromImage(*Image);
      Paint->drawPixmap(0, 0, *Pix);
      Paint->end();
      Berechnung = FALSE;
      ImageReady = TRUE;
   }
}

void DrawWidget::PaletteInterpolate(int Start, int Ende)
//  Farbverlauf zwischen den Farbpaletteneintrag Start und Ende erzeugen
{
int Lauf;
int diff;
int sr,sg,sb,er,eg,eb;  // Farbwerte am Anfang und Ende
int r,g,b;

  diff = (Ende-Start);   // Anzahl der zu ndernden Werte
  if (diff <= 0) return;   // End < Anf
  // Farbanteile am Start
  sr = qRed(ColorMap[Start]);
  sg = qGreen(ColorMap[Start]);
  sb = qBlue(ColorMap[Start]);
  // Farbanteile am Ende
  er = qRed(ColorMap[Ende]);
  eg = qGreen(ColorMap[Ende]);
  eb = qBlue(ColorMap[Ende]);
  // Farben interpolieren
  for (Lauf=Start+1; Lauf<Ende; Lauf++) {
    r = sr + (er-sr)*(Lauf-Start)/diff;
    g = sg + (eg-sg)*(Lauf-Start)/diff;
    b = sb + (eb-sb)*(Lauf-Start)/diff;
    //Image->setColor(Lauf, qRgb(r,g,b));
    ColorMap[Lauf] = qRgb(r,g,b);
  }
}

void DrawWidget::CreatePalette(void)
{
  if (ColorDeep==1) {
     ColorMap[0] = 0;  // Schwarz
     ColorMap[1] = qRgb(255,255,255);  // Weiss
  } else 
  if (ColorDeep==8) {
      //fprintf(stdout, "KMandel::InitPalette()\n");
     /*
     ColorMap[0] = qRgb(0,0,0);//   = 0x00000000;     // schwarz
     ColorMap[31] = qRgb(0,0,0xFF); //  = 0x000000FF;     // blau
     ColorMap[63] = qRgb(0x80, 0x80, 0x80); //  = 0x00800080;     // rot-blau (dunkel)
     ColorMap[95] = qRgb(0xFF,0,0);//  = 0x00FF0000;     // rot
     ColorMap[127] =qRgb(0x80,0x80,0);// = 0x00808000;     // rot-grn (dunkel)
     ColorMap[159]= qRgb(0,0xB0,0);// = 0x0000FF00;     // grn
     ColorMap[191]=qRgb(0,0x80,0x80);// = 0x00008080;     // blau-grn (dunkel)
     ColorMap[223]=qRgb(0x80,0x80,0x80);// = 0x00808080;     // grau
     ColorMap[255]=0;// = 0x00000000;     // schwarz
     */
     ColorMap[0] = qRgb(0,0,0);
     ColorMap[31] = qRgb(0,0,0xFF);
     PaletteInterpolate(0,31);
     ColorMap[63] = qRgb(200,0,200);
     PaletteInterpolate(31,63);
     ColorMap[95] = qRgb(255,0,0);
     PaletteInterpolate(63, 95);
     ColorMap[127] =qRgb(200,200,0);
     PaletteInterpolate(95,127);
     ColorMap[159]= qRgb(0,255,0);
     PaletteInterpolate(127,159);
     ColorMap[191]=qRgb(0,200,200);
     PaletteInterpolate(159,191);
     ColorMap[223]=qRgb(200,200,200);
     PaletteInterpolate(191, 223);
     ColorMap[255]=0;
     PaletteInterpolate(223,255);
   } // else no Palette
}

void DrawWidget::InitPalette(void)
// Farbpalette in das Image-Objekt bertragen
{
unsigned int lauf;
   
   if (ColorDeep > 8) return;  // Truecolor ohne Farbpalette

   for (lauf=0; lauf<NumColors; lauf++)
      Image->setColor(lauf, ColorMap[lauf]);
}

void DrawWidget::NewFractal(int FracType) {
//fprintf(stdout," Fractype = %d\n",FracType);
   AbortCalc();
   Image->fill(0);
   delete Fractal;
//fprintf(stdout,"nach delete Fractal\n");
   switch (FracType) {
      case FT_MANDEL: Fractal = new MandelBrot();
                      break;
      case FT_JULIA:  Fractal = new Julia();
                      break;
      case FT_MLAMBDA:  Fractal = new LMandel();
                      break;
      case FT_JLAMBDA: Fractal = new LJulia();
                      break;
      case FT_INVMANDEL: Fractal = new InvMand();
                      break;
      case FT_GINGER: Fractal = new Ginger();
                      break;
      case FT_GALAXY: Fractal = new Galaxy();
                      break;
      case FT_HOP:    Fractal = new Hop();
                      break;
      case FT_CHIP:   Fractal = new Chip();
                      break;
      case FT_MANDEL_N:   Fractal = new MandelHochN();
                      break;
      case FT_JULIA_N:   Fractal = new JuliaHochN();
                      break;
      case FT_TRIPLE: Fractal = new Triple();
                      break;
      case FT_HENON:  Fractal = new Henon();
                      break;
      case FT_BARNSLEYJA:  Fractal = new BarnsleyJA();
                      break;
      case FT_BARNSLEYB:  Fractal = new BarnsleyB();
                      break;
      case FT_BARNSLEYC:  Fractal = new BarnsleyC();
                      break;
      case FT_ORBIT3:  Fractal = new Orbit3();
                      break;
      case FT_WANKEL:  Fractal = new Wankel();
                      break;
      case FT_QUADRAT:  Fractal = new Quadrat();
                      break;
      case FT_LIAPI:  Fractal = new Liapi();
                      break;
      case FT_STAR:   Fractal = new Star();
                      break;
      case FT_LORENZ: Fractal = new Lorenz();
                      break;
      case FT_GENETIC: Fractal = new Genetic();
                      break;
      case FT_PHEONIX: Fractal = new Pheonix();
                      break;
      case FT_POCKEN: Fractal = new Pocken();
                      break;
      case FT_INSEL:  Fractal = new Insel();
                      break;
      case FT_MAND4:  Fractal = new Mand4();
                      break;
      case FT_MAND4B: Fractal = new Mand4b();
                      break;
      case FT_SEDIMENT: Fractal = new Sediment();
                      break;
      case FT_NEWTON: Fractal = new Newton();
                      break;
      case FT_COMPLEXNEWTON: Fractal = new ComplexNewton();
                      break;
      case FT_LEC211: Fractal = new Lec211();
                      break;
      case FT_LEC212: Fractal = new Lec212();
                      break;
      case FT_LEC213: Fractal = new Lec213();
                      break;
      default:        Fractal = new BaseFract();//sollte nie eintreten!
   }
   Fractal->Init(PicSizeX, PicSizeY);
//fprintf(stdout,"nach fractal->init\n");
   if (kmand)
      kmand->SetStatus(Fractal->GetFracName(), SZ_TYPE);
//fprintf(stdout,"nach setstatus\n");
   update();
//fprintf(stdout,"nach update\n");
}

int DrawWidget::LoadColorMap(const char* MapName)
{
FILE* f;
int r,g,b,Lauf;
char Line[128];

   f=fopen(MapName,"rt");
   if (f==NULL) return FALSE;
   Lauf=0;
   while (!feof(f)) {
      fgets(Line, 128, f);
      sscanf(Line,"%d %d %d",&r,&g,&b);
      //fprintf(stdout,"r=%d g=%d b=%d\n",r,g,b);
      ColorMap[Lauf++] = qRgb(r,g,b);
      if (Lauf >= 256) break;  // mehr als 256 Farben nicht mglich
   }

   fclose(f);
   InitPalette();
   update();
   return TRUE;
}

int DrawWidget::SelectColorMap()
{
   if (ColorDeep>8) {
      QMessageBox::information(this,i18n("No Colormap"),
                      i18n("No Colormap in Hi- or Truecolor Mode."),
                      i18n("OK"),0,0,0);
      return FALSE;
   } else {

      QString f = KFileDialog::getOpenFileName(ColorMapDir.absPath(), "*.map", this );
      if ( !f.isEmpty() ) {
         // the user selected a valid existing file
         return LoadColorMap(f.data());
      } else {
         // the user cancelled the dialog
         return FALSE;
      }
   }
}

void DrawWidget::ChangePicParams(int sx, int sy, int cdeep)
{// Bildparameter ndern
long C;
   AbortCalc();
//fprintf(stdout,"changpicparams: abort\n");
   if (ColorDeep!=cdeep) {
//fprintf(stdout,"Colordeepchange\n");
      switch(cdeep) {
         case 1 : ColorDeep = 1;
                  NumColors = 2;
                  break;
         case 8 : ColorDeep = 8;
                  NumColors = 256;
                  break;
         case 32: ColorDeep = 32;
                  NumColors = 0;
                  break;
         default: ColorDeep=8;
                  NumColors=256;
      }
      CreatePalette();
   }

   SetPicSize(sx,sy);  // neue Gre
   delete Image;
   C=NumColors;
   if ( (C>2) && (C<256) )  C=256;
   Image = new QImage(PicSizeX, PicSizeY, ColorDeep, NumColors,QImage::BigEndian);
   InitPalette();
   update();
}

int DrawWidget::SaveColorMap(void)
{           // Farbpalette speichern
FILE* f;
int Lauf;

QString DateiName = KFileDialog::getSaveFileName(ColorMapDir.absPath(), "*.map", this );
   if ( !DateiName.isEmpty() ) {     // the user gave a file name
      if (DateiName.right(4) != ".map")     // Endung anfgen, falls
         DateiName += ".map";               // keine angegeben wurde
      if (QFile::exists(DateiName.data())){ // Datei existiert bereits
         //if (!QMessageBox::query(i18n("Confirm"),i18n("Overwrite the existing file?")))
         //   return TRUE;
         if (QMessageBox::information(this, i18n("Confirm"),
                i18n("Overwrite the existing file?")+(QString)"\n"+DateiName,
                i18n("OK"),i18n("Abort"), 0, 0, 1) == 1) return TRUE;
      }

      f = fopen(DateiName.data(),"wt");
      if (f==0) return FALSE;         // Fehler beim ffnen

      for (Lauf=0; Lauf <256; Lauf++) {
         fprintf(f, "%d %d %d\n", qRed(ColorMap[Lauf]), qGreen(ColorMap[Lauf]), qBlue(ColorMap[Lauf]));
      }
      fclose(f);
   }
   return TRUE;
}

void DrawWidget::EditColorMap(void)
{
PalEditDlg PalEdit(&(ColorMap[0]), NumColors, 0, i18n("Palette editieren"));
int Lauf;

   if (ColorDeep>8) {
      QMessageBox::information(this,i18n("No Colormap"),
                      i18n("No Colormap in Hi- or Truecolor Mode."),
                      i18n("OK"),0,0,0);
   } else
   if (PalEdit.exec()) {
      for (Lauf=0; Lauf<256; Lauf++) {
         ColorMap[Lauf] = PalEdit.GetColor(Lauf);
         InitPalette();
         update();
      }
   }
}

void DrawWidget::MenuSavePic()
{ // Image speichern
   QString f = KFileDialog::getSaveFileName(MandelDir.absPath(), "*.bmp", this );
   if ( !f.isEmpty() ) {
      if (f.right(4) != ".bmp")     // Endung anfgen, falls
         f += ".bmp";               // keine angegeben wurde
      if (QFile::exists(f.data())){ // Datei existiert bereits
         //if (!QMessageBox::query(i18n("Confirm"),i18n("Overwrite the existing file?")))
         //   return;
         if (QMessageBox::information(this, i18n("Confirm"),
                i18n("Overwrite the existing file?")+(QString)"\n"+f,
                i18n("OK"),i18n("Abort"), 0, 0, 1) == 1) return;
      }

      if (!SaveImage(f.data()))
         QMessageBox::information(this, i18n("Error"),
                                  i18n("Error saving Picture."),
                                  i18n("OK"),0,0,0);
   }
}

int DrawWidget::SaveImage(const char* Dateiname) {
   return Image->save(Dateiname,"BMP");
}

void DrawWidget::LoadPic(const char* DateiName)
// Gespeichertes Bild laden
{
   Image->load(DateiName, "BMP");
   SetPicSize(Image->width(),Image->height());
   ColorDeep = Image->depth();
   ImageReady = TRUE;
   Berechnung = FALSE;
//fprintf(stdout,"Depth=%d, NumColors=%d\n",ColorDeep,NumColors);
//if (Image->bitOrder() == QImage::BigEndian)
//   fprintf(stdout,"Bitorder ok\n");
//else
//   fprintf(stdout,"Bitorder Fehler\n");
}


void DrawWidget::AbortCalc(void)
// Berechnung abbrechen 
{
//fprintf(stdout,"Abortcalc\n");
   Berechnung=FALSE;
   ImageReady=FALSE; 
   Image->fill(0);
}

void DrawWidget::MenuZoomIn()
{ // Reinzoomen
   switch (ZoomMode) {
      case ZOOM_MOVE : if (ZoomRectVisible) {
                          ZoomRectVisible = FALSE;
                          AbortCalc();
                             Fractal->ZoomIn(ZoomX1-ZoomRectX/2,ZoomY1-ZoomRectY/2, 
                                              ZoomX1+ZoomRectX/2,ZoomY1+ZoomRectY/2);
                          update();
                       }
                       break;
      default        : AbortCalc();
                       Fractal->ZoomIn(PicSizeX>>1,PicSizeY>>1,ZoomFactor);
                       update();
   }
}

void DrawWidget::MenuZoomOut()
{ // Rauszoomen
   switch (ZoomMode) {
      case ZOOM_MOVE : if (ZoomRectVisible) {
                          ZoomRectVisible = FALSE;
                          AbortCalc();
                             Fractal->ZoomOut(ZoomX1-ZoomRectX/2,ZoomY1-ZoomRectY/2, 
                                              ZoomX1+ZoomRectX/2,ZoomY1+ZoomRectY/2);
                          update();
                       }
                       break;
      default        : AbortCalc();
                       Fractal->ZoomOut(PicSizeX>>1,PicSizeY>>1,ZoomFactor);
                       update();
   }
}

void DrawWidget::MenuZoomHistory(void) {
// Zoom-History-Funktion aktivieren, Einblenden von Ausschnitten, die bereits als
// Parameterdateien gespeichert sind.
QKeyEvent kev(QEvent::KeyPress, Key_L, 76, 0);
/*   if (ZoomMode == ZOOM_HIST) {
      ZoomMode = ZOOM_NONE;
   } else {
      ZoomMode = ZOOM_HIST;
   }*/
   Taste(&kev);  // Tastedruck 'L' simulieren
}

void DrawWidget::GetParamsBlock(struct MandDataType* MandData)
// Daten setzen, die in der Parameterdatei gespeichert werden sollen
{
   MandData->ZoomFactor = ZoomFactor;
   MandData->ZoomFactorAnimation = ZoomFactorAnimation;
   MandData->ZoomAnimationPics = ZoomAnimationPics;
   MandData->UpdateInterval = UpdateInterval;
   MandData->UpdateBool = UpdateBool;
   MandData->CalcMode = CalcMethod;
   MandData->IterIncrement = IterIncrement;
   MandData->ZoomHistFactor = ZoomHistFactor;
   MandData->OrbitDisplayMode = OrbitDisplayMode;
}

void DrawWidget::SetParamsBlock(struct MandDataType* MandData)
// Daten setzen, die aus einer Parameterdatei gelesen wurden
{
   ZoomFactor = MandData->ZoomFactor;
   ZoomFactorAnimation = MandData->ZoomFactorAnimation;
   ZoomAnimationPics = MandData->ZoomAnimationPics;
   UpdateInterval = MandData->UpdateInterval;
   UpdateBool = MandData->UpdateBool;
   CalcMethod = MandData->CalcMode;
   IterIncrement = MandData->IterIncrement;
   ZoomHistFactor = MandData->ZoomHistFactor;
   OrbitDisplayMode = MandData->OrbitDisplayMode;
}

void DrawWidget::Fertig()
{
   fprintf(stdout,"FERTIG %s",(ImageReady?"ok\n":"Fehler\n"));
}

void DrawWidget::PicToColorMap(void) {
// Farbpalette aus dem gelesenem Bild benutzen
unsigned int lauf;

   if (ColorDeep > 8) return;  // Truecolor ohne Farbpalette

   for (lauf=0; lauf<NumColors; lauf++)
      ColorMap[lauf] = Image->color(lauf);
}

void DrawWidget::FillRect(int X, int Y, int B, int H, long Col) {
// Rechteckigen Ausschnitt fllen
// Patameter: X,Y linke obere Ecke, B,H Gre   Col Farbe
int n,i;

//printf("Fillrect x=%d y=%d b=%d h=%d PicX=%d\n",X,Y,B,H,PicSizeX);
   if ((H<1) || (B<1)) return;
   if (X>=PicSizeX) return;
   if (X+B>=PicSizeX) B=PicSizeX-X;
   if (Y>=PicSizeY) return;
   if (Y+H>=PicSizeY) H=PicSizeY-Y;
   
   for (n=Y;n<Y+H;n++)
      for (i=X;i<X+B;i++) {
         //printf("Putpix x=%d y=%d\n",i,n);
         PutPixel(i,n,Col);
      }
//printf("Fillrect Ende\n");         
}

void DrawWidget::MarkPoints(int xl, int yl, int xh, int yh) { // fr FillQuad
int n,t,dix,diy,xw;
double di,sx;

//printf("MarkPoint xl=%d yl=%d xh=%d yh=%d\n",xl,yl,xh,yh);
   if (yl>yh) {
      t=yl; yl=yh; yh=t;
      t=xl; xl=xh; xh=t;
   }
   dix = xh-xl;
   diy = yh-yl;
   if (diy != 0)
      di = 1.0*dix/diy;
   else
      di = 0.0;
   sx = xl;
   if (yl<miny) miny=yl;
   if (yh>maxy) maxy=yh;
   for (n=yl; n<=yh; n++) {
      xw=(int)(sx+di*(n-yl));
      if (xw<0) xw=0;
      if (tst[n][0]!=0) {
         if (xw<tst[n][0])
            tst[n][0]=xw;
            
      } else
         tst[n][0]=xw;
      if (xw>tst[n][1]) tst[n][1]=xw;
      if (tst[n][0]<0) tst[n][0]=0;
      if (tst[n][1]>PicSizeX-1) tst[n][1]=PicSizeX-1;
   }
}

void DrawWidget::FillQuad(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int Color) {
// Fllt ein unregelmssiges Viereck mit den
// Eckpunkten (x1y1)(x2y2)(x3y3)(x4y4) und Farbe Color
// Alle Winkel mssen "positiv" sein!
int n,le;

//printf("fillquad Start Picx=%d\n",PicSizeX);
   miny=1000;
   maxy=0;
   memset(tst, 0, sizeof(tst));
   MarkPoints(x1,y1,x2,y2);
   MarkPoints(x2,y2,x3,y3);
   MarkPoints(x3,y3,x4,y4);
   MarkPoints(x4,y4,x1,y1);
   if (miny<0) miny=0;
   for (n=miny; n<=maxy; n++) {
      if ((tst[n][0]!=0) && (tst[n][1]!=0)) {
         le=abs(tst[n][1]-tst[n][0]);
         le++;
         if (tst[n][0]>=PicSizeX-1){
//printf("fillquad Ende1\n");
             return;
         } 
         FillRect(tst[n][0],n,le,1,Color);
      }
   }
//printf("fillquad Ende PicX=%d\n",PicSizeX);
}

void DrawWidget::MenuZoomMode() {
int p_active;

   switch (ZoomMode) {
      case ZOOM_NONE : ZoomMode = ZOOM_MOVE; break;
      case ZOOM_MOVE : ZoomMode = ZOOM_ANIMATION; 
                       // Zoomrechteck entfernen, falls sichtbar
      case ZOOM_HIST : if (ZoomRectVisible) {
                          p_active = Paint->isActive();
                          if (!p_active)
                             Paint->begin(this);
                          Paint->drawPixmap(zxa,zya,*Pix, zxa,zya, bra,hoa);
                          if (!p_active)
                             Paint->end();
                          ZoomRectVisible=FALSE;
                       }
                       break;
      case ZOOM_ANIMATION: if (ZoomRectVisible) {
                          p_active = Paint->isActive();
                          if (!p_active)
                             Paint->begin(this);
                          Paint->drawPixmap(zxa-6,zya-6,*Pix, zxa-6,zya-6, 12,12);
                          if (!p_active)
                             Paint->end();
                          ZoomRectVisible=FALSE;
                       }
                       //ZoomMode = ZOOM_FAST; 
                       ZoomMode = ZOOM_NONE; 
                       break;
/*      case ZOOM_FAST : ZoomMode = ZOOM_NONE;
                       break;
*/
   }
   kmand->SetStatus(ZoomNames[ZoomMode], SZ_ZOOM);
}

void DrawWidget::ZoomAnimation(int X, int Y) {
ZoomAnimDlg ZADlg(this, X, Y, this, "ZADlg");

   AbortCalc();
//   ImageReady=TRUE;
   ZADlg.show();
}

void DrawWidget::ZoomHistBuildList(void) {
QDir Path;
int Res;
unsigned int Lauf;
ParameterFile *Par;
struct MandDataType MD;
double cx,cy;           // Mitte des (Parameter-)Bildes
double s;               // Grenfaktor
struct ZoomHistType *Neu;   // neues Element der History-Liste

   if (ZoomHistStart!=NULL)
      ZoomHistDeleteList();
      
   Path = MandelDir;
   Path.setNameFilter("*.par");   // nur Parameterdateien
   for (Lauf=0; Lauf<Path.count(); Lauf++) {
      Par = new ParameterFile(Path.absPath()+"/"+Path[Lauf]);
      Res = Par->Open(PAR_READ);
      Res = Par->BlockSize('M','U');
      if (Res >= 0) {
         Res = Par->ReadBlock('M','U', &MD, Res);
         if (Res >= 0) {  // lesen ok
            if (MD.FractalType == Fractal->GetFracType()) { // typen sind gleich
               //printf("OK\n");
               cx = (MD.li+MD.re)*0.5;
               cy = (MD.un+MD.ob)*0.5;
               if ((cx>=Fractal->li) && (cx<=Fractal->re) && 
                   (cy<=Fractal->ob) && (cy>=Fractal->un)) 
               {   // Ausschnitt ist sichtbar
                  s = (Fractal->re - Fractal->li)/(MD.re - MD.li);
                  //printf("SICHTBAR! %7.4f\n",s);
                  if ((s<ZoomHistFactor) && (s>1.0)) {
                     // Ausschnitt hat greren Zoomfaktor und ist nicht
                     // zu stark vergrert
                     //printf("BERNAHME!\n");
                     Neu = new ZoomHistType;
                     Neu->Next = NULL;
                     Neu->Prev = NULL;
                     sprintf(Neu->FileName, "%s",Par->FileName());
                     //printf("Datei %s\n",Neu->FileName);
                     Neu->x1 = (int)(PicSizeX*(MD.li-Fractal->li)/(Fractal->re-Fractal->li));
                     Neu->x2 = (int)(PicSizeX*(MD.re-Fractal->li)/(Fractal->re-Fractal->li));
                     Neu->y1 = (int)(PicSizeY*(MD.un-Fractal->un)/(Fractal->ob-Fractal->un));
                     Neu->y2 = (int)(PicSizeY*(MD.ob-Fractal->un)/(Fractal->ob-Fractal->un));
                     
                     // Einfgen in die Liste
                     if (ZoomHistEnd==NULL) {  // erstes Element
                        ZoomHistStart = Neu;
                        ZoomHistEnd = Neu;
                     } else {
                        ZoomHistEnd->Next = Neu;  // Nachfolger
                        Neu->Prev = ZoomHistEnd;  // Vorgnger
                        ZoomHistEnd = Neu;        // Zeiger auf das neue Ende setzen
                     }
                  }
               }
            }
         }
      }
      Par->Close();
      delete Par;
      //printf("Filename=%s\n",Path[Lauf]);
   }
   ZoomHistList = ZoomHistStart;
   if (ZoomHistStart != NULL)
      ZoomHistShowRects();
   else {  // keine weiteren Details gefunden
      ZoomMode = ZOOM_NONE;
      QMessageBox::information(this,i18n("No more Details"),
                      i18n("No more details for zoom history"),
                      i18n("OK"),0,0,0);
   }
   
/*   // Testausgabe
   Neu=ZoomHistStart;
   printf("Vorwrts\n");
   while (Neu!=NULL) {
      printf("%4d %4d %4d %4d %s\n", Neu->x1,Neu->x2,Neu->y1,Neu->y2,Neu->FileName);
      Neu=Neu->Next;
   }
   Neu=ZoomHistEnd;
   printf("Rckwrts\n");
   while (Neu!=NULL) {
      printf("%4d %4d %4d %4d %s\n", Neu->x1,Neu->x2,Neu->y1,Neu->y2,Neu->FileName);
      Neu=Neu->Prev;
   }*/
}

void DrawWidget::ZoomHistDeleteList(void){
ZoomHistType *Ptr,*Help;

printf("ZoomHistDeleteList\n");
   Ptr=ZoomHistStart;
   while (Ptr!=NULL) {
      Help = Ptr->Next;
      delete Ptr;
      Ptr = Help;
   }
   ZoomHistStart=NULL;
   ZoomHistEnd=NULL;
   ZoomHistList=NULL;
   kmand->SetStatus("", SZ_HIST);
}

void DrawWidget::ZoomHistShowRects(void){
ZoomHistType *Ptr;
int p_active;
char *SPtr;

printf("ZoomHistShowRects\n");
   if (ZoomHistList==NULL) return;
   
   Ptr=ZoomHistStart;
   while (Ptr!=NULL) {
      p_active = Paint->isActive();
      if (!p_active)
         Paint->begin(this);
      if (Ptr==ZoomHistList)
         Paint->setPen(ZoomRectColors[2*ZoomRectColorSet+1]);
      else
         Paint->setPen(ZoomRectColors[2*ZoomRectColorSet]);
      Paint->drawRect(Ptr->x1,Ptr->y1, Ptr->x2-Ptr->x1,Ptr->y2-Ptr->y1);
      if (!p_active)
         Paint->end();

      Ptr = Ptr->Next;
   }
   SPtr = rindex(ZoomHistList->FileName, '/')+1;
   kmand->SetStatus(SPtr, SZ_HIST);
}

void DrawWidget::WriteParams(QString* DN)
// Speichert die Fraktalparameter in eine Datei. Der Dateiname
// wird in DN bergeben und sollte schon die gewnschte Endung .par
// haben
{
ParameterFile* Par;
struct MandDataType MandData;
int Res,S;
char Info[] = "Fractal created by Mandel Ultimativ V 0.6 (c) CFJH";

   Fractal->GetParamsBlock(&MandData); // Frak-Parameter speichern
   GetParamsBlock(&MandData);          // Gen-Parameter speichern

   // Parameter mit dem neuen Parameter-Objekt speichern
   Par = new ParameterFile(*DN);
   Res = Par->Open(PAR_WRITE);
   if (Res == PAR_OK) {
      Par->WriteBlock('M','U', &MandData, sizeof(MandData));
      if (SaveColorMapBool)
         Par->WriteBlock('C','M', &(ColorMap), sizeof(ColorMap));
      S = Fractal->GetSpecialParamsBytes();
//      fprintf(stdout,"Special Params: Size =%d\n",S);
      if (S>0)
         //fprintf(stdout,"ptr=%d\n",Fractal->GetSpecialParamsBlock());
         Par->WriteBlock('T','S', Fractal->GetSpecialParamsBlock(),S);
      Par->WriteBlock('I','N', &Info, sizeof(Info));
      Par->Close();
   }
   delete Par;
   if (Res!=PAR_OK) {
      QMessageBox::information(this, i18n("Error"),
                               i18n("Parameter File Writing Error"),
                               i18n("OK"),0,0,0);
      return;
   }
}

void DrawWidget::AddHist(void)
// aktuelle Parameter in der Zoomhist speichern
{
struct HistType *Neu;
int S;
void *DataPtr;

   //debug ("DrawWidget::AddHist(), Type=%s",Fractal->GetFracName());
   
   Neu = (HistType*)malloc(sizeof(struct HistType));
   if (!Neu) {
      debug("AddHist:Kein Speicher, 'Neu'");
      return;
   }
   Neu->Next = NULL;
   Neu->Prev = NULL;
   Neu->SpecialParams=NULL;
   Neu->SpecialSize=0;
   Neu->Data = (MandDataType*)malloc(sizeof(struct MandDataType));
   if (!Neu->Data) {
      debug("AddHist: Kein Speicher, 'Data'");
      free(Neu);
      return;
   }
   GetParamsBlock(Neu->Data);
   Fractal->GetParamsBlock(Neu->Data);
   
   S=Fractal->GetSpecialParamsBytes();
   if (S>0) {  // Fraktal hat special Params
      Neu->SpecialParams = malloc(S);
      if (!Neu->SpecialParams) {
         debug("AddHist: Kein Speicher, 'SpecialParams'");
         free(Neu->Data);
         free(Neu);
         return;
      }
      DataPtr=Fractal->GetSpecialParamsBlock();
      memcpy(Neu->SpecialParams, DataPtr, S);
      Neu->SpecialSize=S;
   }
   if (HistList==NULL) {  // erstes Element
      HistList = Neu;
      HistStart= Neu;
      HistEnd  = Neu;
   } else {  // In die Liste einfgen
      if (HistList->Next) DeleteHist(HistList); // nachfolgende lschen
      Neu->Prev = HistList;
      HistList->Next = Neu;
      HistList = Neu;
   }
   
/*   
   //testausgabe
   Neu=HistStart;
   while (Neu!=NULL) {
      debug("MandType=%d",Neu->Data->FractalType);
      Neu=Neu->Next;
   }
   debug("HL-MandType=%d",HistList->Data->FractalType);
   */
}

void DrawWidget::HistBack() {
//struct HistType *Neu;
void *DataPtr;

   if (HistList->Prev) {
      HistList=HistList->Prev;
      AbortCalc();
      ImageReady=TRUE;
      qApp->processEvents();
      NewFractal(HistList->Data->FractalType);
      SetParamsBlock(HistList->Data);
      Fractal->SetParamsBlock(HistList->Data);
      if (HistList->SpecialSize>0) {  // Special Params vorhanden
         DataPtr=Fractal->GetSpecialParamsBlock();
         memcpy(DataPtr, HistList->SpecialParams, HistList->SpecialSize);
         Fractal->PutSpecialParamsBlock();
      }
      update();
   }
   //testausgabe
/*
   debug("testaus");
   Neu=HistStart;
   while (Neu!=NULL) {
      debug("MandType=%d",Neu->Data->FractalType);
      Neu=Neu->Next;
   }
   debug("HL-MandType=%d",HistList->Data->FractalType);*/
}

void DrawWidget::HistForward() {
void *DataPtr;

   if (HistList->Next) {
      HistList=HistList->Next;
      AbortCalc();
      ImageReady=TRUE;
      qApp->processEvents();
      NewFractal(HistList->Data->FractalType);
      SetParamsBlock(HistList->Data);
      Fractal->SetParamsBlock(HistList->Data);
      if (HistList->SpecialSize>0) {  // Special Params vorhanden
         DataPtr=Fractal->GetSpecialParamsBlock();
         memcpy(DataPtr, HistList->SpecialParams, HistList->SpecialSize);
         Fractal->PutSpecialParamsBlock();
      }
      update();
   }
}

void DrawWidget::DeleteHist(HistType *AbHier)
// Lscht alle Eintrge, die NACH AbHier kommen
{
HistType *P, *D;
//   debug ("DrawWidget::DeleteHist(...)");
   if (AbHier->Next==NULL) return;  // bereits letztes Element
   
   P = AbHier->Next;
   while (P) {
      free(P->Data);
      if (P->SpecialParams) free(P->SpecialParams);
      if (P->Next) D=P->Next; else D=NULL;
      free(P);
      P=D;
   }
   AbHier->Next=NULL;
}

void DrawWidget::InitParameterRec(struct MandDataType* MD)
{
int Lauf;

   // Version 0.2a
   MD->FractalType = 0;
   MD->li = -3.0;
   MD->re =  3.0;
   MD->un = -2.0;
   MD->ob =  2.0;
   MD->MaxIter = 256;
   MD->BailOut = 4.0;
   MD->ConvMode = 0;
   for (Lauf=0;Lauf<16;Lauf++) {
      MD->p[Lauf] = 0.0;
      MD->l[Lauf] = 0;
   }
   // Version 0.2b
   MD->ZoomFactor = 5.0;
   MD->ZoomFactorAnimation = 1.03;
   MD->ZoomAnimationPics = 5;
   MD->UpdateInterval = 50;
   MD->UpdateBool = TRUE;
   // V0.21
   MD->InsideMode = INS_SOLIDCOLOR;
   MD->OutsideMode = OUT_ITER;
   MD->InsideColor = 0;
   MD->OutsideColor = 128;
   MD->PotentialSlope = 500.0;
   MD->CalcMode = CM_LINE;
   // V0.4
   strcpy(MD->FileMask, "zm%0.6d");
   MD->FileCount = 0;
   MD->ColorMode = COL_LINEAR;
   MD->LogFirstColor = 0;
   MD->LogShift = 0.04;
   MD->AutoLogColor=FALSE;
   MD->IterIncrement=3;
   // V0.41
   MD->ZoomHistFactor = 100.0;
   // V0.6
   MD->OrbitDisplayMode = 0;
}

/*bool DrawWidget::eventFilter( QObject *, QEvent * qev) {

   if (qev->type() == Event_Paint) {
      debug("eventHandler - Resize");
      return NoResize;
   }
   return FALSE;
}*/

void DrawWidget::DrawRect(int x,int y,int b,int h) {
int Lauf;

   for (Lauf=x; Lauf<=x+b; Lauf++) {
      PutPixel(Lauf,y,255);
      PutPixel(Lauf,y+h,255);
   }
   for (Lauf=y; Lauf<y+h; Lauf++) {
      PutPixel(x,Lauf,255);
      PutPixel(x+b,Lauf,255);
   }
}

void DrawWidget::ConvertColors(void) {
// Farben remappen.
// Fr MakeJourney wird eine Farbe gebraucht, um die Rechtecke darzustellen.
// Es wird die Farbe 255 (8-Bit Modus) auf 'Wei' gesetzt, alle regulr
// vorkommenden Punkte mit Farbindex 255 werden auf Index 254 gesetzt,
// in der Hoffnung, da man es nicht sieht.
int x,y;

   for (x=0; x<Image->width(); x++)
   for (y=0; y<Image->height(); y++)
   if (GetPixel(x,y) == 255)
      PutPixel(x,y,254);
   Image->setColor(255,qRgb(255,255,255));  // 'Wei' setzen   
}

void DrawWidget::RePaintImage(void){        // Bild neu darstellen, fr IAO-Tpen
   if (UpdateBool) {
      Pix->convertFromImage(*Image, QPixmap::Auto);
      Paint->drawPixmap(0, 0, *Pix);
   }
   qApp->processEvents();
}


KMandel::KMandel(const char *name )
        : KMainWindow( 0,name )
{
QString Datei;
int i;

    //Btn = new QPushButton("Hallo",this,"Btn");
    //Btn->resize(250,200);
    
    ZoomRectColors[0] = "white";
    ZoomRectColors[1] = "red";
    ZoomRectColors[2] = "darkGreen";
    ZoomRectColors[3] = "yellow";
    ZoomRectColors[4] = "cyan";
    ZoomRectColors[5] = "green";

    InitStatusZeile();

    Client = new DrawWidget(this);
    setCentralWidget(Client);
    //Client->resize(150,100);

    InitMenue();
    InitToolBar();

    //SetClientSize(MAXX, MAXY);
    // SetKeys();
    Config = kapp->config();
    Config->setGroup("Allgemeines");
    Datei = Config->readEntry("StartParameter","");
    if (!Datei.isEmpty())
       LoadParamFile(&Datei);
    MandelDir = Config->readEntry("MandelDir","");
    ColorMapDir = Config->readEntry("ColorMapDir","");
    i = Config->readNumEntry("NoParamWarn", 0);
    Client->NoParamWarn = (i==1);
    i = Config->readNumEntry("SaveColorMap", 0);
    Client->SaveColorMapBool = (i==1);
    
    Client->AddHist();
//fprintf(stdout,"kmandel init ende\n");
}

KMandel::~KMandel()
{
   Client->AbortCalc();
   //if (StatusZeile) delete StatusZeile;
   //if (TBar) delete TBar;
   //if (MainMenu) delete MainMenu;
}

void KMandel::SetClientSize(int Breite, int Hoehe)
// Gre des Clientbereiches durch ndern der Gre des
// Frames auf die angegebene Gre bringen
{
   int DiffX,DiffY;
   int NewX, NewY;
   
// fprintf(stdout, "SetClientSize: width()=%d height()=%d\n",Client->width(),Client->height());
   DiffX = Client->width()-Breite;
   DiffY = Client->height()-Hoehe;
// fprintf(stdout, "SetClientSize: DiffX=%d DiffY=%d\n",DiffX,DiffY);
   NewX  = width()-DiffX;
   NewY  = height()-DiffY;
// fprintf(stdout, "SetClientSize: width=%d height=%d\n",width(),height());
// fprintf(stdout, "SetClientSize: NewX=%d NewY=%d\n",NewX,NewY);
   resize(NewX, NewY);

   if (Client->height() != Hoehe) {
      // "nachtriggern"
      fprintf(stdout,"SetClientSize: Hhe nicht erreicht\n");
      DiffY = Client->height()-Hoehe;
      NewY  = height()-DiffY;
      resize(NewX, NewY);
   }
   if (Client->height() != Hoehe) {
      // "nachtriggern"
      fprintf(stdout,"SetClientSize: Hhe nicht erreicht\n");
      DiffY = Client->height()-Hoehe;
      NewY  = height()-DiffY;
      resize(NewX, NewY);
   }
//   qApp->processEvents();
// fprintf(stdout, "SetClientSize: Ende\n");
}

void KMandel::InitMenue()
// Initialisierung des Hauptmens
{
    //fprintf(stdout,"KMandel::InitMenue\n");

    // Datei-Men
    QPopupMenu* FileMenu = new QPopupMenu; 
    FileMenu->insertItem(i18n("&New"), this, SLOT(MenuNew()),CTRL+Key_N);
    FileMenu->insertItem(i18n("&Open"), this, SLOT(MenuOpen()), CTRL+Key_O );
    FileMenu->insertItem(i18n("&Save Pic"),Client,SLOT(MenuSavePic()) );
    FileMenu->insertItem(i18n("Save &Params"), this, SLOT(MenuWriteParams()), CTRL+Key_S );
    FileMenu->insertItem(i18n("&Filesettings"), this, SLOT(MenuFileSettings()) );
    FileMenu->insertSeparator();
    FileMenu->insertItem(i18n("&Exit"), qApp, SLOT(quit()), CTRL+Key_Q );

    // Fraktal-Men
    QPopupMenu* FraktalMenu = new QPopupMenu;
    FraktalMenu->insertItem(i18n("&Type"), this, SLOT(ShowFracTypeDlg()),Key_F5 );
    FraktalMenu->insertItem(i18n("&Parameter"), this, SLOT(ShowFracParamsDlg()), Key_F6 );
    FraktalMenu->insertItem(i18n("Type-&specific"), this , SLOT(MenuSpecialParams()), Key_F7 );
    FraktalMenu->insertItem(i18n("Parameteranim"), this , SLOT(MenuParameterAnim()), Key_F8 );

    // Special
    QPopupMenu* SpecialMenu = new QPopupMenu;
    SpecialMenu->insertItem(i18n("&Make Journey"), this, SLOT(MenuJourney()), ALT+Key_J );

    // Farbpalette
    QPopupMenu* PicMenu = new QPopupMenu;
    PicMenu->insertItem(i18n("&General Settings"), this, SLOT(MenuPicSettings()), Key_F9 );
    PicMenu->insertItem(i18n("&Edit Palette"), this, SLOT(MenuEditPal()),Key_F10 );
    PicMenu->insertItem(i18n("&Load Palette"), this, SLOT(MenuLoadColorMap()),Key_F11 );
    PicMenu->insertItem(i18n("&Save Palette"), this, SLOT(MenuSaveColorMap()),Key_F12 );

    // Zooming
    QPopupMenu* ZoomMenu = new QPopupMenu;
    ZoomMenu->insertItem(i18n("Zoom &in"), Client, SLOT(MenuZoomIn()), Key_Plus );
    ZoomMenu->insertItem(i18n("Zoom &out"), Client, SLOT(MenuZoomOut()), Key_Minus );
    ZoomMenu->insertItem(i18n("&History"), Client, SLOT(MenuZoomHistory()) );
    ZoomMenu->insertItem(i18n("Zoom &Setup"), this, SLOT(MenuZoomSettings()), Key_Asterisk );

    // ber-Dialoch und Hilfe
    HelpMenu = new KHelpMenu( this, aboutData, FALSE );
    //QPopupMenu* MUMenu = new QPopupMenu;
    //MUMenu->insertItem(i18n("&Help"), this, SLOT(MenuHelp()), Key_F1 );
    //MUMenu->insertSeparator();
    //MUMenu->insertItem(i18n("&About MU"), this, SLOT(ShowAboutDlg()) );
    //MUMenu->insertItem(i18n("About &KDE"), this, SLOT(ShowAboutKDEDlg()));

    MainMenu = new KMenuBar(this);
    MainMenu->insertItem(i18n("&File"), FileMenu, MID_FILE);
    MainMenu->insertItem(i18n("Fra&ctal"), FraktalMenu);
    MainMenu->insertItem(i18n("&Picture"), PicMenu);
    MainMenu->insertItem(i18n("&Special"), SpecialMenu);
    MainMenu->insertItem(i18n("&Zoom"), ZoomMenu);
    MainMenu->insertSeparator();
    //MainMenu->insertItem(i18n("&Help"), MUMenu);
    MainMenu->insertItem(i18n("&Help"), (QPopupMenu*)(HelpMenu->menu()) );
    //MainMenu->show();
    //setMenu(MainMenu);
}

void KMandel::HelpContents() {
  if (HelpMenu)
    HelpMenu->appHelpActivated();
}

void KMandel::InitStatusZeile()
{
   StatusZeile = new KStatusBar(this, "StatusZeile");
   StatusZeile->insertFixedItem("XXXXXXXXXXXXXXX", SZ_TYPE);
   StatusZeile->changeItem("Mandelbrot", SZ_TYPE);
   if (Opt_StatusbarBug) {
     StatusZeile->insertFixedItem("fertigx", SZ_PROGRESS+1000);
     StatusZeile->changeItem("", SZ_PROGRESS+1000);
   } else {
     StatusZeile->insertFixedItem("fertigx", SZ_PROGRESS);
     StatusZeile->changeItem("", SZ_PROGRESS);
   }
   StatusZeile->insertFixedItem("XXXXXXX", SZ_ZOOM);
   StatusZeile->changeItem("normal", SZ_ZOOM);
   StatusZeile->insertFixedItem("XXXXXXXXXXXX", SZ_HIST);
   StatusZeile->changeItem("", SZ_HIST);
   StatusZeile->insertItem("", 4711, TRUE);
   //setStatusBar(StatusZeile);
}

void KMandel::InitToolBar(void)
{
   TBar = toolBar("MainToolBar");
   
#define I(f,o,s,h) TBar->insertButton( BarIcon(f),\
	 0, SIGNAL(clicked()), o, SLOT(s()), TRUE, h);

   // File
   I( "filenew",      this,     MenuNew,         i18n("Reinit this Type"))
   I( "fileopen",     this,     MenuOpen,        i18n("Open"))
   I( "filesave",     this,     MenuWriteParams, i18n("Save Params"))
   TBar->insertSeparator();//ToolBar->insertSeparator();
   I( "back",         Client,   HistBack,        i18n("Back"))
   I( "forward",      Client,   HistForward,     i18n("Forward"))
   TBar->insertSeparator();//ToolBar->insertSeparator();
   // Fractal
   I( "filenew",      this,     ShowFracTypeDlg, i18n("New Fractal"))
   I( "contents",     this,     ShowFracParamsDlg, i18n("Generic Parameters"))
   I( "attach",       this,     MenuSpecialParams, i18n("Special Parameters"))
   // I( "anim",         this,     MenuParameterAnim, i18n("Parameteranim"))
   TBar->insertSeparator();//ToolBar->insertSeparator();
   //Zoom
   I( "viewmag+",     Client,   MenuZoomIn,      i18n("Zoom in"))
   I( "viewmag-",     Client,   MenuZoomOut,     i18n("Zoom out"))
   I( "viewmag",      Client,   MenuZoomMode,    i18n("Zoommode"))
   TBar->insertSeparator();//ToolBar->insertSeparator();
   // Palette
   I( "colorize",     this,     MenuEditPal,     i18n("Edit Palette"))
   I( "fileopen",     this,     MenuLoadColorMap,i18n("Load Palette"))
   I( "filesave",     this,     MenuSaveColorMap,i18n("Save Palette"))
   //ToolBar->insertSeparator();//ToolBar->insertSeparator();
   //I( "help",         this,     MenuHelp,        i18n("Help!"))

   //addToolBar(ToolBar);
}

void KMandel::ShowAboutDlg() {
   //fprintf(stdout, "ShowAboutDlg\n");
   AboutDlg About(0,"AboutDlg");
   About.exec();
}

void KMandel::ShowAboutKDEDlg() {
   QMessageBox::information(this, i18n("About KDE"),
      "The KDE Desktop Environment was written by the KDE Team,\na world-wide network of software engineers committed to\nfree software development.\n\nVisit http://www.kde.org for more information on the KDE Project.\nPlease consider joining and supporting KDE.\n\nPlease report bugs at http://buglist.kde.org.",
      i18n("Ok"));
}

void KMandel::ShowFracTypeDlg() {
   FracTypeDlg FracType(0,"FracTypeDlg");
   FracType.SetFracType(Client->Fractal->GetFracType());
   if (FracType.exec()) {
      //fprintf(stdout,"Dialog ok\n");
      Client->NewFractal(FracType.GetFracType());
      Client->AddHist();
   } //else
      //fprintf(stdout,"Dialog cancel\n");
}

void KMandel::ShowFracParamsDlg()
{
const char* Strg;
   FracParamsDlg FracParams(  Client, 0, "Fraktalparameter");
//   FracParams.SetCornerParams(Client->Fractal->li,
//                              Client->Fractal->un,
//                              Client->Fractal->re,
//                              Client->Fractal->ob);
//   FracParams.SetGeneralParams(Client->Fractal->MaxIter);
//   FracParams.SetConvParams(  Client->Fractal->ConvMode,
//                              Client->Fractal->BailOut);
//   FracParams.SetParamParams( &Client->Fractal->p[0]);
//   FracParams.SetInfoParams(  Client->Fractal->GetFracType(),
//                              Client->Fractal->GetFracName(),
//			      Client->Fractal->GetFracInfo());
   if (FracParams.exec()) {
      // fprintf(stdout,"Dialog ok\n");
      Client->AbortCalc();
      //if (!
      Strg=FracParams.SetParams();
      if (Strg) fprintf(stdout,"Parameter-Fehler : %s\n", Strg);
//      FracParams.GetCornerParams(&Client->Fractal->li,
//                                 &Client->Fractal->un,
//                                 &Client->Fractal->re,
//                                 &Client->Fractal->ob); //)
//       //fprintf(stdout," Param-Error\n");
//      FracParams.GetGeneralParams(&Client->Fractal->MaxIter);
//      FracParams.GetConvParams(  &Client->Fractal->ConvMode,
//                                 &Client->Fractal->BailOut);
//      FracParams.GetParamParams( &Client->Fractal->p[0]);
      Client->Fractal->ReInit();
      Client->update();
      Client->AddHist();
      // printf("Berechnung=%d\nImageready=%d\nLoadImage=%d\n",Client->Berechnung,Client->ImageReady,Client->LoadImage);
   } //else
     // fprintf(stdout,"Dialog cancel\n");
}

void KMandel::MenuNew()
{
   Client->NewFractal(Client->Fractal->GetFracType());  // reinit
   Client->AddHist();
}

void KMandel::MenuLoadColorMap()
{
   Client->SelectColorMap();  // reinit
}

void KMandel::MenuWriteParams()
{
QString DateiName;
//ParameterFile* Par;
//struct MandDataType MandData;
//int Res,S;
//char Info[] = "Fractal created by Mandel Ultimativ V 0.4 (c) CFJH";

   DateiName = KFileDialog::getSaveFileName(MandelDir.absPath(), "*.par", 0, "Parameter speichern");
   if (DateiName.isEmpty()) return;  // Dialog Cancel

   if (DateiName.right(4) != ".par") {
      DateiName += ".par";
      //fprintf(stdout,"keine Endung '.par'\n");
      //fprintf(stdout,"Endung jetzt >%s<\n",DN.data());
   }

   if (QFile::exists(DateiName.data())){ // Datei existiert bereits
      //if (!QMessageBox::query(DateiName,i18n("Overwrite the existing file?")))
      //   return;
      if (QMessageBox::information(this, i18n("Confirm"),
             i18n("Overwrite the existing file?")+(QString)"\n"+DateiName,
             i18n("OK"),i18n("Abort"), 0, 0, 1) == 1) return;
   }
   Client->WriteParams(&DateiName);
/*
   Client->Fractal->GetParamsBlock(&MandData); // Frak-Parameter speichern
   Client->GetParamsBlock(&MandData);          // Gen-Parameter speichern

   // Parameter mit dem neuen Parameter-Objekt speichern
   Par = new ParameterFile(DateiName);
   Res = Par->Open(PAR_WRITE);
   if (Res == PAR_OK) {
      Par->WriteBlock('M','U', &MandData, sizeof(MandData));
      if (Client->SaveColorMapBool)
         Par->WriteBlock('C','M', &(Client->ColorMap), sizeof(Client->ColorMap));
      S = Client->Fractal->GetSpecialParamsBytes();
//      fprintf(stdout,"Special Params: Size =%d\n",S);
      if (S>0)
         //fprintf(stdout,"ptr=%d\n",Client->Fractal->GetSpecialParamsBlock());
         Par->WriteBlock('T','S', Client->Fractal->GetSpecialParamsBlock(),S);
      Par->WriteBlock('I','N', &Info, sizeof(Info));
      Par->Close();
   }
   delete Par;
   if (Res!=PAR_OK) {
      QMessageBox::information(this, i18n("Error"),
                               i18n("Parameter File Writing Error"),
                               i18n("OK"),0,0,0);
      return;
   }*/
}

void KMandel::MenuOpen()
{
QString DN,D2;

   // Dateiname erfragen
   DN = KFileDialog::getOpenFileName(MandelDir.absPath(), "*.par", this);
   if (DN.isEmpty()) return;   // Cancel im Dialoch

   if (DN.right(4) != ".par") {
      DN += ".par";
      //fprintf(stdout,"keine Endung '.par'\n");
      //fprintf(stdout,"Endung jetzt >%s<\n",DN.data());
   }

   LoadParamFile(&DN);
   Client->AddHist();
   return;
}

void KMandel::MenuHelp()
{
//   invokeHTMLHelp("","");
//KApplication::getKApplication()->invokeHTMLHelp("","");
//fprintf(stdout,"Help called...\n");
   kapp->invokeHTMLHelp("","");
}

void KMandel::MenuSpecialParams()
{
   if (Client->Fractal->SetupFrac(Client->Fractal)) {
      Client->AbortCalc();
      Client->Fractal->ReInit();
      Client->update();
      Client->AddHist();
   }
}

void KMandel::MenuParameterAnim()
{
ParameterAnimDlg PADlg(Client, this, "PADlg");

   printf("parameteranim\n");
   Client->AbortCalc();
   Client->ImageReady=TRUE;  // "Bild ferig" vorgaukeln
   //connect(Client, SIGNAL(FertigSig()), &PADlg, SLOT(Fertig()));
   printf("connect...\n");
   PADlg.exec();
}

void KMandel::MenuPicSettings()
{ // generelle Bildeinstellung
PicSetDlg PsDlg(Client, 0, "Picturesettings");

   if (PsDlg.exec())      // Dialog ok
      if(PsDlg.SetValues())
         SetClientSize(Client->GetSizeX(),Client->GetSizeY());
}

void KMandel::MenuSaveColorMap()
{ // Farbpalette speichern
   if (Client->ColorDeep>8) {
      QMessageBox::information(this,i18n("No Colormap"),
                      i18n("No Colormap in Hi- or Truecolor Mode."),
                      i18n("OK"),0,0,0);
   } else {
      if (!Client->SaveColorMap())
         QMessageBox::information(this, i18n("Error"),
                                  i18n("Error saving Colormap"),
                                  i18n("OK"),0,0,0);
   }
}

void KMandel::MenuEditPal()
{ // Farbpalette bearbeiten
   Client->EditColorMap();
}

void KMandel::resizeEvent( QResizeEvent * qre)
{
// fprintf(stdout,"KMandel::resizeEvent\n");
KMainWindow::resizeEvent(qre);
}

void KMandel::MenuZoomSettings()
{ // Zoomeinstellungen
ZoomSettingsDlg ZoomSettings(Client, this, "ZoomSettings");

   if (ZoomSettings.exec()) {
      ZoomSettings.SetValues();
   }
}

void KMandel::MenuFileSettings()
{ // Dateiparameter einstellen
FileSettingsDlg FileSettings(Client, this, "FileSettings");

   if (FileSettings.exec()) {
      FileSettings.SetValues();
   }
}

void KMandel::SetStatus(const char* Text, int ID)
{
//debug("Setstaus >%s<",Text);
//  if (ID!=SZ_PROGRESS)
    StatusZeile->changeItem(Text, ID);
}

/*void KMandel::SetKeys(void)
{
	kKeys->registerWidget("KMandel", this);

#define AKCF(f,k,o,s)	kKeys->addKey(f, k);\
			kKeys->connectFunction( "KMandel", f, o, SLOT(s()));
   // File
   AKCF(i18n("&New"),          "CTRL+N",       this,   MenuNew       )
   AKCF("Open",         "CTRL+O",       this,   MenuOpen      )
   AKCF("Save Params",  "CTRL+S",       this,   MenuWriteParams)
   AKCF("Quit",	        "ALT+X",        qApp,   quit          )
   // Factal
//   AKCF("FracType",     "F5",           this,   ShowFracTypeDlg)
   AKCF("FracParams",   "F6",           this,   ShowFracParamsDlg)
   AKCF("FracSpecials", "F7",           this,   MenuSpecialParams)
   // Palette
   AKCF("PalGeneral",   "F9",           this,   MenuPicSettings)
   AKCF("PalEdit",      "F10",          this,   MenuEditPal)
   AKCF("PalLoad",      "F11",          this,   MenuLoadColorMap)
   AKCF("PalSave",      "F12",          this,   MenuSaveColorMap)
   // Zoom
   AKCF("Zoom in",      "Plus",	        Client, MenuZoomIn    )
   AKCF("Zoom out",     "Minus",        Client, MenuZoomOut   )
   AKCF("History",      "",             Client, MenuZoomHistory)
   AKCF("ZoomSettings", "Asterisk",     this,   MenuZoomSettings)
   // Hilfe
   AKCF("Help",         "F1",           this,   MenuHelp      )

#undef AKCF

//	config->setGroup( "KMandel" );
}*/

int  KMandel::LoadParamFile(QString* DN, int Auto)
{
int Res, Size;
ParameterFile* Par;
struct MandDataType MandData;

   Client->AbortCalc();
   Client->ImageReady=TRUE;
   //if (!Auto) 
   qApp->processEvents();
   Client->InitParameterRec(&MandData);

   // Header der Parameterdatei lesen
   Par = new ParameterFile(DN->data());
   Res = Par->Open(PAR_READ);
   if (Res != PAR_OK) {  // Fehler beim ffnen
      delete Par;
      if (!Auto)
         QMessageBox::information(this, i18n("Error"),
                                  i18n("File Open Error"),
                                  i18n("OK"),0,0,0);
      return FALSE;
   }
   // Blockgre ermitteln
   Size = sizeof(MandData);
   Res = Par->BlockSize('M','U');
   if (Res < 0) {
      if (!Auto)
         QMessageBox::information(this, i18n("Error"),
                                  i18n("No Mandel Ultimativ Parameterfile"),
                                  i18n("OK"),0,0,0);
      Par->Close();
      delete Par;
      return FALSE;
   }
   if (!Auto) {
      if (Res < Size) {  // Weniger Daten, frhere Version
         if (!Client->NoParamWarn)
            QMessageBox::information(this, i18n("Notice"),
                                     i18n("Parameterfile created with older programversion"),
                                     i18n("OK"),0,0,0);
      } else
      if (Res > Size) {  // Mehr Daten, neuere Dateiversion
         if (!Client->NoParamWarn)
            QMessageBox::information(this, i18n("Notice"),
                                     i18n("Parameterfile created with newer programversion"),
                                     i18n("OK"),0,0,0);
         Res = Size;     // Anzahl der zu lesenden Zeichen begrenzen
      }
   }
//fprintf(stdout,"vor Fraktalparameter lesen\n");
   // Fraktalparameter lesen
   Res = Par->ReadBlock('M','U',&MandData, Res);
//fprintf(stdout,"nach ReadBlock\n");
   //Client->AbortCalc();
   Client->NewFractal(MandData.FractalType);
//fprintf(stdout,"nach NewFractal\n");
   Client->Fractal->SetParamsBlock(&MandData);// Fraktal-Parameter setzen
//fprintf(stdout,"nach SetParamsBlock\n");
   Client->SetParamsBlock(&MandData);         // Sonstige Parameter setzen
//fprintf(stdout,"nach Fraktalparameter lesen\n");

   Res = Client->Fractal->GetSpecialParamsBytes();
   if (Res > 0) {
//      fprintf(stdout,"Lese Special-Params, %d\n",Res);
      Par->ReadBlock('T','S', Client->Fractal->GetSpecialParamsBlock(), Res);
      Client->Fractal->PutSpecialParamsBlock(); // SpecParams initialisieren
   }

   // Farbpalette lesen, falls vorhanden
   Res = Par->ReadBlock('C','M', &(Client->ColorMap), sizeof(Client->ColorMap));
   if (Res == PAR_OK)
      Client->InitPalette();

   Par->Close();
   delete Par;

   if (Auto) {
      Client->update();
      return TRUE;
   }
   
   // Bild suchen und laden, falls vorhanden
   DN->remove(DN->length()-4, 4);  // Endung ".par" entfernen
   DN->append(".bmp");
   if (QFile::exists(DN->data())) {
      // zugehriges Bild gefunden, laden und anzeigen
      Client->LoadImage = TRUE;
      StartUp = TRUE;
      Client->LoadPic(DN->data());
      SetClientSize(Client->GetSizeX(), Client->GetSizeY());
      switch (Client->ColorDeep) {
         case  1 : Client->NumColors = 2;  break;
         case  8 : Client->NumColors = 256; break;
         case 32 : Client->NumColors = 0; break;
         default:fprintf(stdout,"Unbekannte Farbtiefe %d!", Client->ColorDeep);
      }

      Client->LoadPic(DN->data());
      Client->PicToColorMap();     // Farbpalette aus dem Bild benutzen
      Client->Berechnung = FALSE;
      Client->ImageReady = TRUE;
      printf("Startup=%d\n",StartUp);
      StartUp = FALSE;
   } else
      Client->update();  // Neuberechnung

   return TRUE;
}

void KMandel::keyPressEvent (QKeyEvent* kev) {
//debug("keyPressEvent");
   Client->Taste(kev);
}

void KMandel::MenuJourney () {
JourneyDlg D(Client, this, i18n("Make Journey"));
   debug("MenuJourney");
   D.exec();
}

int main( int argc, char **argv )
{
int Res;
   StartUp = TRUE;
   NoResize=FALSE;
fprintf(stdout,"MU-START\n");
   kmand = NULL;
//    KApplication MandelApp = KApplication( argc, argv );
//    MandelApp MApp = MandelApp(argc, argv, "Mandel Ultimativ");

  aboutData= new KAboutData( "kmandel", I18N_NOOP("Kmandel"),
    version, description, KAboutData::License_GPL,
    "(c) 2000, Jrgen Hochwald");
  aboutData->addAuthor("Jrgen Hochwald",0, "jh@jhochwald.de");
  //aboutData->addAuthor("Adam Riese",0, "adam@riese.de");

  KCmdLineArgs::init( argc, argv, aboutData );
  KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.

  // if (kmand==NULL)  fprintf(stdout,"kmand=NULL\n");
  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
  Opt_StatusbarBug=1;
  if (args->isSet("s")) {
    Opt_StatusbarBug=0;
  }
  if (Opt_StatusbarBug) {
    debug ("Statusbar is buggy (compiled option is ON), no progress display");
    debug ("Use Opton -s to switch it on, program may crash on endless updating!");
  }
  MApp = new MandelApp(argc, argv, "kmandel");
  kmand = new KMandel("KMandel");  // Hauptfenster

  MApp->setMainWidget( kmand );
//fprintf(stdout,"vor resize\n");
  kmand->resize(500,449);
//fprintf(stdout,"nach resize\n");
//fprintf(stdout, "nach kmand->resize\n");
  kmand->show();
  kmand->setCaption("Mandel Ultimativ");
  StartUp = FALSE;
//fprintf(stdout,"nach setcaption\n");
//    kmand->SetClientSize(500, 375);// Initial-Size
//fprintf(stdout,"nach setclientsize\n");

  Res = MApp->exec();
printf("nach exec\n");
  return Res;
}
