// Fraktaltyp Newton
// l[0] = Exponent
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <klocale.h>
#include "newton.h"
#include "komplex.h"

void Newton::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   l[0]=4;
   SetSize(SizeX, SizeY);
}

void Newton::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2;     // Gre des Fraktals
   *x2 =  2;
   *y1 = -1.5;
   *y2 =  1.5;
}

double Newton::SetDefaultParams(int)
{
   return 0.0;
}

int Newton::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;
int iter=0,Lauf;
struct komplex h,h2,n,z,za;
   
   z.re=rx;  z.im=ry;
   do {
      za=z;
      // Berechnung z^(p-1)
      h=z;
      for (Lauf=2; Lauf<l[0]; Lauf++) {
         h=cmul(h,z);
         //h=n;
      }
      n.re=(double)l[0]*h.re;
      n.im=(double)l[0]*h.im;
      
      h2=cmul(h,z);
      
      h.re=(double)(l[0]-1)*h2.re+1.0;
      h.im=(double)(l[0]-1)*h2.im;
      
      z=cdiv(h,n);
      
      h=csub(z,za);
      iter++;
   } while ( (iter<MaxIter) && ( fabs(h.re*h.re+h.im*h.im)>0.000001));
   return PostCompute(iter,h.re,h.im);
}

const char* Newton::GetFracInfo(void)
{
   return "Newton Nullstellensuche";
}

int Newton::SetupFrac(BaseFract* Frac) // type specific-setup
{
NewtonDlg MDlg(Frac, 0, "Newton settings");

   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
}

// -----------------------------------------

NewtonDlg::NewtonDlg(BaseFract* Frac, QWidget* parent, const char* name):
          QDialog(parent,name, TRUE)
{
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(i18n("Newton Order"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ExponentSpin = new QSpinBox(this, "OffsetX");
   ExponentSpin->setGeometry(100,50,125,22);
   ExponentSpin->setRange(3,20);
   //ExponentSpin->setEditable(TRUE);
   ExponentSpin->setValue(Frac->l[0]);

   QPushButton* OkBtn = new QPushButton(i18n("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(i18n("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(i18n("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int NewtonDlg::SetSpecificParams(void)
{

   Fractal->l[0] = ExponentSpin->value();
   return TRUE;
}

void NewtonDlg::SetDefaults()
{
   ExponentSpin->setValue(Fractal->l[0]);
}
