// Fraktaltyp Pocken
// l[0] Anzahl der Punkte (50)
// l[1] Art (0=Pocken, 1=Gauss)
// p[0] Intensity

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <klocale.h>
#include "mymath.h"
#include "pocken.h"
//#include "pocken.moc"
#define MAXPOINTS 500
#define TYPEPOCKEN 0
#define TYPEGAUSS 1
// #define DKARTE 2

double PockenPar[2] = {1, 0.0};

struct PointType {
          float X,Y;
	  float Value;
       };

struct PointType Points[MAXPOINTS];

void Pocken::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   l[0] = 50;
   l[1] = TYPEPOCKEN;
   p[1] = 7.5;//ausprobieren!
   InitPocken();
   SetSize(SizeX, SizeY);
}

void Pocken::InitPocken(void) {
int i;
//char Ort[20];

// fprintf(stdout,"initpocken\n");
   Avg=0.0;
   /*switch (l[1]) {
      case DKARTE :   {
                      FILE * f;
                      
                         i=0;
                         f=fopen("wd_18990323.asp","rt");
                         if (!f) break;
                         do {
                            fscanf(f, "%f %f %f %s\n",&Points[i].X,&Points[i].Y,&Points[i].Value,Ort);
                            printf("%f %f %f %s\n",Points[i].X,Points[i].Y,Points[i].Value,Ort);
                            Avg += Points[i].Value;
                            i++;
                         } while (!feof(f));
                         fclose(f);
                         l[0]=i;
                      }
                      break;
      default : */      for (i=0;i<l[0];i++) {
                         Points[i].X = frand();
                         Points[i].Y = frand();
                         Points[i].Value = frand()*2.0-1.0;
                         Avg+=Points[i].Value;
                      }
                      /*break;
   }*/
   Avg/=l[0];  // Mittelwert
   printf("%7.3f\n",Avg);
}

void Pocken::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 =  0;     // Gre des Fraktals
   *x2 =  1;
   *y1 =  0;
   *y2 =  1;
}

double Pocken::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return PockenPar[Index];
}

int Pocken::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;
int Lauf;
double r,entf,hilf;

   r = 0.0;
   switch (l[1]) {
      case TYPEPOCKEN : for (Lauf=0; Lauf<l[0]; Lauf++) {
                           /* Entferung bestimmen */
                           entf = sqrt( sqr(rx-Points[Lauf].X) + sqr(ry-Points[Lauf].Y) );
                           if (entf > 0.00005) {
                              hilf = Points[Lauf].Value * p[0] / entf;
                              r = r+hilf;
                           }
                        }
                        break;
      //case DKARTE :
      case TYPEGAUSS :  for (Lauf=0; Lauf<l[0]; Lauf++) {
                           // Entferung bestimmen
                           entf = p[1]*sqrt( sqr(rx-Points[Lauf].X) + sqr(ry-Points[Lauf].Y) );
                           hilf = Points[Lauf].Value * p[0] *exp(-(entf*entf));
                           r    = r+128.0*hilf;
                        }
                        // Bereich zentrieren und begrenzen
                        r = r + 128.0 - Avg;  // Durchschnittswert beachten
                        if (r<0.0) r=0.0;
                        if (r>255.0) r=255.0;
                        break;
   }
//   fprintf(stdout,"x=%d  y=%d   ret=%7.3f\n",x,y,r);
   return (int)r;
}

const char* Pocken::GetFracInfo(void)
{
   return "?";
}

int Pocken::SetupFrac(BaseFract* Frac) // type specific-setup
{
PockenDlg MDlg(Frac, 0, "Pocken settings");
int h;
   if (MDlg.exec()) {
      h = MDlg.SetSpecificParams();
      InitPocken();
      return h;
   } else
      return FALSE;
}

int Pocken::GetSpecialParamsBytes(void)
{
   return sizeof(Points);
}

void* Pocken::GetSpecialParamsBlock(void)
// Pointer auf Params, lesen
{
   return &Points;
}

void Pocken::PutSpecialParamsBlock(void)
{  // dummy
}


// -----------------------------------------

PockenDlg::PockenDlg(BaseFract* Frac, QWidget* parent, const char* name):
          QDialog(parent,name, TRUE)
{
char Strg[20];

   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(i18n("Pocks Parameter"), this, "Label0");
   Label0->setGeometry(30,5,150,20);

   PointsSpin = new QSpinBox(this, "TransformSpin");
   PointsSpin->setGeometry(125,30,50,20);
   PointsSpin->setRange(5,500);
   PointsSpin->setValue(Fractal->l[0]);
   //PointsSpin->setEditable(TRUE);

   TypeCombo = new QComboBox(this, "TypeCombo");
   TypeCombo->setGeometry(125,60,100,20);
   TypeCombo->insertItem("Pocken");
   TypeCombo->insertItem("Gauss");
   //TypeCombo->insertItem("DE-Karte");
   TypeCombo->setCurrentItem(Fractal->l[1]);

   ScaleEdit = new QLineEdit(this, "ScaleEdit");
   ScaleEdit->setGeometry(125,90,100,22);
   muprintf(Strg,"%0.14f", Fractal->p[0]);
   ScaleEdit->setText(Strg);


   QLabel *Label1 = new QLabel(i18n("#Pocks"), this, "Label1");
   Label1->setGeometry(20,30,90,20);
   
   QLabel *Label2 = new QLabel(i18n("Type"), this, "Label2");
   Label2->setGeometry(20,60,90,20);
   
   QLabel *Label3 = new QLabel(i18n("Color Scale"), this, "Label3");
   Label3->setGeometry(20,90,90,20);
   

   QPushButton* OkBtn = new QPushButton(i18n("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(i18n("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(i18n("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int PockenDlg::SetSpecificParams(void)
{
char* eptr;
double p0;

   p0 = strtod(ScaleEdit->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->l[0] = PointsSpin->value();
   Fractal->l[1] = TypeCombo->currentItem();
   Fractal->p[0] = p0;
   return TRUE;
}

void PockenDlg::SetDefaults()
{
char Strg[20];

   PointsSpin->setValue(50);
   TypeCombo->setCurrentItem(TYPEPOCKEN);
   muprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ScaleEdit->setText(Strg);
}

