// Fraktaltyp Sediment
// Parameter
// l0 : DlaType
// l1 : Nachbarn
// l2 : Colorcycle
// l3 : 

#define DLA 0
#define SEDIMENT 1
#define SEDIMENT2 2
#define NACHBAR4 0
#define NACHBAR8 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <klocale.h>
#include "mymath.h"
#include "sediment.h"


double SedimentPar[2] = {1, 0.0};

extern KMandel *kmand; 

void Sediment::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
   l[0] = DlaType = DLA;
   l[1] = Nachbar = NACHBAR8;
   l[2] = ColorCycle = 33;
   ReInit();
}

void Sediment::ReInit(void){
   //fprintf(stdout,"Sediment.reinit\n");
   Frame = 3;
   Color = 128;
} 

void Sediment::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 =  0;     // Gre des Fraktals
   *x2 =  1;     // (unwichtig fr IAO-Typen)
   *y1 =  0;
   *y2 =  1;
}

double Sediment::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return SedimentPar[Index];
}


void Sediment::InitIAO(void)
{
int i;

   MaxX = PicSizeX-1;
   MaxY = PicSizeY-1;
   Color = 127;
   
   switch (DlaType) {
      case DLA : // Start in der Mitte mit einem einzigen Punkt
		 Links  = (MaxX >> 1) - Frame;
		 Rechts = (MaxX >> 1) + Frame;
		 Oben   = (MaxY >> 1) - Frame;
		 Unten  = (MaxY >> 1) + Frame;
		 kmand->Client->PutPixel(MaxX >> 1, MaxY >> 1, Color);  // Kristallisationskeim in der Mitte
		 break;
      case SEDIMENT : // ganze untere Zeile ist Start
      case SEDIMENT2 :
		 Links = 1;
		 Rechts = MaxX-1;
		 Unten = MaxY-1;
		 Oben = MaxY-Frame;
		 for (i=Links; i<=Rechts; i++)     // Startpunkte
		    kmand->Client->PutPixel(i,MaxY, Color);
		 break;
   }
}

void Sediment::InitPoint(void) {
   switch (DlaType) {
      case DLA :  switch (rand() % 4) {
		     case 0 :  // linker Rand
			      PosX = Links;
			      PosY = Oben + rand() % (Unten-Oben);
			      break;
		     case 1 :  // rechter Rand
			      PosX = Rechts;
			      PosY = Oben + rand() % (Unten-Oben);
			      break;
		     case 2 :  // oberer Rand
			      PosY = Oben;
			      PosX = Links + rand() % (Rechts-Links);
			      break;
		     case 3 :  // unterer Rand
			      PosY = Unten;
			      PosX = Links + rand() % (Rechts-Links);
			      break;
		  }  break;
      case SEDIMENT :
		  PosY = Oben;
		  PosX = Links + rand() % (Rechts-Links);
		  break;
      case SEDIMENT2 :
		  PosY = 1;
		  PosX = PicSizeX >> 1;
		  break;
   }
}

int Sediment::Kgefunden(int PosX, int PosY) {
   if (kmand->Client->GetPixel(PosX-1,PosY) != 0) return TRUE;  // direkte Nachbarn
   if (kmand->Client->GetPixel(PosX+1,PosY) != 0) return TRUE;
   if (kmand->Client->GetPixel(PosX,PosY-1) != 0) return TRUE;
   if (kmand->Client->GetPixel(PosX,PosY+1) != 0) return TRUE;
   if (Nachbar == NACHBAR4) return FALSE;
   if (kmand->Client->GetPixel(PosX-1,PosY-1) != 0) return TRUE;  // diagonale Nachbarn
   if (kmand->Client->GetPixel(PosX-1,PosY+1) != 0) return TRUE;
   if (kmand->Client->GetPixel(PosX+1,PosY-1) != 0) return TRUE;
   if (kmand->Client->GetPixel(PosX+1,PosY+1) != 0) return TRUE;
   return FALSE;
}

void Sediment::NextPos(void) {
// nchste Position bestimmen, dabei den Rand bercksichtigen

   switch (DlaType) {
      case DLA : switch (rand() % 4) {
		    case 0 :  // nach links
			     PosX--;
			     if (PosX<Links) PosX=Rechts;
			     break;
		    case 1 :  // rechts
			     PosX++;
			     if (PosX > Rechts) PosX = Links;
			     break;
		    case 2 :  // oben
			     PosY--;
			     if (PosY < Oben) PosY = Unten;
			     break;
		    case 3 :  // unten
			     PosY++;
			     if (PosY > Unten) PosY = Oben;
			     break;
		 }
		 break;
      case SEDIMENT :
      case SEDIMENT2 :
		 switch (rand() % 3) {
		    case 0 : // links
		             PosX--;
                             if (PosX<Links) PosX=Rechts;
                             break;
		    case 1 : // rechts
		             PosX++;
		             if (PosX > Rechts) PosX = Links;
		             break;
		    case 2 : PosY++;
		 }
		 break;
   }
   //putpixel(PosX,PosY,32); // malen
}

int Sediment::AdjustFrame(void) {
// Rahmen korrigieren, immer "Frame" Punkte entfernt
// Rckgabe "TRUE", wenn Rahmen Bildrand erreicht
   if (Links+Frame  > PosX) Links  = PosX-Frame;
   if (Rechts-Frame < PosX) Rechts = PosX+Frame;
   if (Oben+Frame   > PosY) Oben   = PosY-Frame;
   if (Unten-Frame  < PosY) Unten  = PosY+Frame;
   if (Links < 1) Links = 1;
   if (Rechts > MaxX-1) Rechts = MaxX-1;
   if (Oben < 1) Oben = 1;
   if (Unten > MaxY-1) Unten = MaxY-1;
   if (FinishOnFrame) {
      switch (DlaType) {
	 case DLA : if (Links==1) return TRUE;        // linker Rand erreicht
		    if (Rechts==MaxX-1) return TRUE;  // rechter ...
		    if (Oben==1) return TRUE;         // oberer
		    if (Unten==MaxY-1) return TRUE;   // unterer
		    break;
	 case SEDIMENT :
	 case SEDIMENT2 :
		    if (Oben < Frame) return TRUE;
		    break;
      }
   }
   return FALSE;
}

void Sediment::Diffusion(void) {
int Lauf;   // Durchlufe
int SameColor;   // Anzahl der Punkte mit gleicher Farbe
int ColorMax;    // Anzahl der Punkte gleicher Farbe

   MaxX = PicSizeX; // -1  Wenn FinishOnFrame=FALSE, dann dir Grenzen um einen
   MaxY = PicSizeY; // -1  Pixel kleiner als die Bildgre, sonst Laufzeitfehler

   Color = 16;
   SameColor = 0;
   Frame = 3;
   FinishOnFrame = TRUE;
   ColorMax = (int)(ColorCycle*(1.0*MaxX*MaxY)/200000.0);
   ReInit();

//debug("DlaType=%d",DlaType);
//debug("Nachbar=%d",Nachbar);
//debug("ColorCycle=%d",ColorCycle);

   PGMax = (int)(0.005*MaxIter*MaxX*MaxY);
   PGCount=0;
   for (Lauf=0; Lauf < (int)(0.005*MaxIter*MaxX*MaxY); Lauf++) {
      PGCount++;
      // Startpunkt bestimmen
      InitPoint();

      while (!Kgefunden(PosX,PosY)) {
	 NextPos();
      }
      if (AdjustFrame()) break;  // Rand erreicht, dann fertig
      if (kmand->Client->ColorDeep==1)
         kmand->Client->PutPixel(PosX, PosY, 1);
      else {
         kmand->Client->PutPixel(PosX, PosY, Color);
         SameColor++;
         if (SameColor >= ColorMax) {
            kmand->Client->ShowProgress();

            SameColor = 0;
	    Color = (Color+1);
	    if (kmand->Client->ColorDeep == 8)
	       Color %= 256;
	 }
      }

      if ((Lauf % 100) == 0) {
         if ((Lauf % 500) == 0) {
            if (DlaType==SEDIMENT || DlaType==SEDIMENT2) {
               if ((Lauf % 2000)==0)
               kmand->Client->RePaintImage();
            } else
               kmand->Client->RePaintImage();
         }
         qApp->processEvents();
      }
      if (!kmand->Client->Berechnung) {
         debug("BREAK");
         break;
      }
   }
}


void Sediment::CalcIAO(void)
{
   Diffusion();
   return;
}

const char* Sediment::GetFracInfo(void)
{
   return  "Sediment Flow";
}

int Sediment::SetupFrac(BaseFract* Frac) // type specific-setup
{
SedimentDlg MDlg(Frac, 0, "Sediment settings");
int h;
   if (MDlg.exec()) {
      h = MDlg.SetSpecificParams();
      DlaType=l[0];
      Nachbar=l[1];
      ColorCycle=l[2];
      ReInit();
      return h;
   } else
      return FALSE;
}

long Sediment::GetProgress(void) {
int a,l,r,o,u;

   switch (DlaType) {
      case DLA : r = (100*(PicSizeX-Rechts))/PicSizeX;
                 u = (100*(PicSizeY-Unten))/PicSizeY;
                 l = (100*Links)/PicSizeX;
                 o = (100*Oben)/PicSizeY;
                 a = o;
                 if (a>u)  a = u;
                 if (a>r)  a = r;
                 if (a>l)  a = l;
                 return 100-a*2;
      case SEDIMENT :
      case SEDIMENT2 :
                 return 100-(100*Oben)/PicSizeY;
   }
   return 0;
}

// -----------------------------------------

SedimentDlg::SedimentDlg(BaseFract* Frac, QWidget* parent, const char* name):
          QDialog(parent,name, TRUE)
{

   Fractal = Frac;
   setFixedSize(300,250);
   setCaption(Fractal->GetFracName());

   QLabel *Lab = new QLabel(i18n("Sediment Parameters"), this, "Lab");
   Lab->setGeometry(30,5,150,20);

   TypeCombo = new QComboBox(this, "TypeCombo");
   TypeCombo->setGeometry(130,35,125,20);
   TypeCombo->insertItem(i18n("Dla"));
   TypeCombo->insertItem(i18n("Sediment"));
   TypeCombo->insertItem(i18n("Sediment Linear"));
   TypeCombo->setCurrentItem(Fractal->l[0]);

   NachbarCombo = new QComboBox(this, "NachbarCombo");
   NachbarCombo->setGeometry(130,65,125,20);
   NachbarCombo->insertItem("4");
   NachbarCombo->insertItem("8");
   NachbarCombo->setCurrentItem(Fractal->l[1]);

   ColorSpin = new QSpinBox(this, "ColorSpin");
   ColorSpin->setGeometry(130,95,50,20);
   ColorSpin->setRange(1,1000);
   ColorSpin->setValue(Fractal->l[2]);
   //ColorSpin->setEditable(TRUE);

   QLabel *Label1 = new QLabel(i18n("Sedimenttype:"), this, "Label1");
   Label1->setGeometry(20,35,90,20);
   QLabel *Label2 = new QLabel(i18n("Nachbarn:"), this, "Label2");
   Label2->setGeometry(20,65,90,20);
   QLabel *Label3 = new QLabel(i18n("Color cycle:"), this, "Label3");
   Label3->setGeometry(20,95,90,20);
   

   QPushButton* OkBtn = new QPushButton(i18n("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,200,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(i18n("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,200,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(i18n("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,200,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int SedimentDlg::SetSpecificParams(void)
{

   Fractal->l[0] = TypeCombo->currentItem();
   Fractal->l[1] = NachbarCombo->currentItem();
   Fractal->l[2] = ColorSpin->value();
   return TRUE;
}

void SedimentDlg::SetDefaults()
{
   TypeCombo->setCurrentItem(DLA);
   NachbarCombo->setCurrentItem(NACHBAR8);
   ColorSpin->setValue(33);
}

