#ifndef SEDIMENT_H
#define SEDIMENT_H
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbt.h>
#include <qdialog.h>
#include <qlined.h>
#include <qchkbox.h>
#include <qspinbox.h>
#include "basefrac.h"
#include "kmandel.h"

typedef int FeldTyp[129][129];

class Sediment : public BaseFract
{
public:
   virtual void Init(int SizeX, int SizeY);
   virtual void ReInit(void);
   virtual void SetDefaultCorners(double *x1, double *x2, double *y1, double *y2);
   virtual int GetFracClass(void) { return FC_IAO; }; // FracKlasse
   virtual double SetDefaultParams(int);
   virtual void InitIAO(void);
   virtual void CalcIAO(void);
   virtual const int GetFracType() { return FT_SEDIMENT; }
   virtual const char* GetFracName(void) { return "Sediment"; }
   virtual const char* GetFracInfo(void);    // Informationen ber dieses Fraktal
   virtual int SetupFrac(BaseFract* Frac); // type specific-setup
   virtual long GetProgress(void);
   
private:
   void InitPoint(void);
   int Kgefunden(int PosX, int PosY);
   void NextPos(void);
   int AdjustFrame(void);
   void Diffusion(void);

   int Links,Rechts,Oben,Unten;  // Rand-Bereich
   int DlaType;                  // Type
   int Nachbar;                  // Art der Nachbaraschaftsbeziehung (4 oder 8)
   int Frame;                    // Abstand des Kristalls zum Randbereich
   int ColorCycle;               // Farbnderung(-wechsel)
   int Color;                    // aktuelle Farbe
   int PosX,PosY;                // Position des Teilchens
   int MaxX, MaxY;               // Bildgre
   int FinishOnFrame;            // Beenden, wenn erster Punkt des Bildrand erreicht
   long PGMax, PGCount;           // fr Fortschrittsanzeige
};

class SedimentDlg : public QDialog
{
Q_OBJECT
public:
   SedimentDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0);
   int SetSpecificParams(void);

private slots:
   void SetDefaults();

private:
   BaseFract* Fractal;   // Pointer to the Fractal
   QLineEdit *ScaleEdit;
   QSpinBox *ColorSpin;
   QComboBox *TypeCombo, *NachbarCombo;
};

#endif
