// Fraktaltyp Wankel
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include <klocale.h>
#include "wankel.h"

double WankelPar[2] = {0, 0.0};

void Wankel::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void Wankel::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2.0;     // Gre des Fraktals
   *x2 =  2.0;
   *y1 = -1.5;
   *y2 =  1.5;
}

double Wankel::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return WankelPar[Index];
}

int Wankel::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;
double h,zr,zi,tr,ti,pr,pi;;
int    iter=0;

  zr=rx; zi=ry;
  pr = p[0];  pi=p[1];
  do {
    tr = zr; ti = zi;    // temp = z

    h= zr*zr-zi*zi-pr;   // z = z + zp
    zi = 2.0*zr*zi-pi;
    zr=h;

    pr = tr;             // zp = temp
    pi = ti;
    iter++;
   } while ((iter<MaxIter) && ConvTest(zr*zr,zi*zi) );

   return PostCompute(iter,zr,zi);
}

const char* Wankel::GetFracInfo(void)
{
   return 
"Formula: z=Pixel;  p=(p0,p1);\n\
loop {\n\
   temp=z\n\
   z=z^2+p\n\
   p=temp\n\
until (|z|>bail)\n\n\
parameters: initial p";
}

int Wankel::SetupFrac(BaseFract* Frac) // type specific-setup
{
/*WankelDlg MDlg(Frac, 0, "Wankel settings");

   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
*/
GenericDlg MDlg(Frac, 0, "Wankel settings");

   MDlg.SetHeader(i18n("Image Constant Values"));
   MDlg.SetParameterNum(2);
   MDlg.SetParameterText(0, "P1");
   MDlg.SetParameterText(1, "P2");
   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------
